﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de acción. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IAccionController : IBaseController<Accion, int>
    {
    }

    /// <summary>
    ///  Controlador de acción. Hereda del controlador base e implementa la interfaz de controlador de acción.
    /// </summary>
    [Authorize]
    public class AccionController : BaseController, IAccionController
    {
        private const int FilasPorPagina = 10;                  // Número de filas por página 
        private readonly IAccionService srvAccion;              // Interfaz del servicio de datos de acciones
        private readonly IActividadService srvActividad;        // Interfaz del servicio de datos de actividades
        private readonly ITematicaService srvTematica;          // Interfaz del servicio de datos de tematicas
        private readonly IOrganizacionService srvOrganizacion;  // Interfaz del servicio de datos de organizaciones

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public AccionController()
        //{
        //    this.srvAccion = new AccionService();
        //    this.srvTematica = new TematicaService().GetAll();
        //    this.srvActividad = new ActividadService().GetAll();
        //    this.srvOrganizacion =  new OranizacionService().GetAll();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvAccion">Interfaz de servicio de acción</param>
        /// <param name="srvTematica">Interfaz de servicio de tematica</param>
        /// <param name="srvActividad">Interfaz de servicio de actividad</param>
        /// <param name="srvOrganizacion">Interfaz de servicio de organización</param>
        /// <returns></returns>
        public AccionController(IAccionService srvAccion, ITematicaService srvTematica,
                                IActividadService srvActividad, IOrganizacionService srvOrganizacion)
        {
            this.srvAccion = srvAccion;
            this.srvTematica = srvTematica;
            this.srvActividad = srvActividad;
            this.srvOrganizacion = srvOrganizacion;
        }

        #region IAccionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de acción
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de acciones</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdAccion", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var acciones = new AccionViewModel
                               {
                                   Acciones = srvAccion.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvAccion.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(acciones);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva acción
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la acción por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea una vista del modelo para editar el objeto nuevo
            var acciones = new AccionViewModelEdit
                               {
                                   Accion = new Accion(),
                                   Tematicas = srvTematica.ObtenerTodos(),
                                   Actividades = srvActividad.ObtenerTodos(),
                                   Organizaciones = srvOrganizacion.ObtenerTodos()
                               };

            // Se definen las propiedades por defecto
            acciones.Accion.IdOrganizacion = 1; // Por defecto la primera organización
            acciones.Accion.IdTematica = 1; // Por defecto la primera tematica
            acciones.Accion.IdActividad = 1; // Por defecto la primera actividad
            acciones.Accion.Activo = true; // Por defecto estará activa 

            // Devuelve una vista para añadir una nueva acción
            return View(acciones);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de una nueva acción
        /// </summary>
        /// <param name="entity">Acción actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Accion")] Accion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var acciones = new AccionViewModelEdit
                                   {
                                       Accion = entity,
                                       Tematicas = srvTematica.ObtenerTodos(),
                                       Actividades = srvActividad.ObtenerTodos(),
                                       Organizaciones = srvOrganizacion.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(acciones);
            }

            // Actualiza los datos
            if (!srvAccion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una acción
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la acción</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Accion accion = srvAccion.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (accion == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var acciones = new AccionViewModelEdit
                               {
                                   Accion = accion,
                                   Tematicas = srvTematica.ObtenerTodos(),
                                   Actividades = srvActividad.ObtenerTodos(),
                                   Organizaciones = srvOrganizacion.ObtenerTodos()
                               };

            // Devuelve una vista para editar
            return View(acciones);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la acción
        /// </summary>
        /// <param name="entity">Acción actual</param>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Accion")] Accion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var acciones = new AccionViewModelEdit
                                   {
                                       Accion = entity,
                                       Tematicas = srvTematica.ObtenerTodos(),
                                       Actividades = srvActividad.ObtenerTodos(),
                                       Organizaciones = srvOrganizacion.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(acciones);
            }

            // Actualiza los datos
            if (!srvAccion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una acción a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param> 
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la acción, sino muestra una vista con el error
            if (!srvAccion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}