﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de actividad. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IActividadController : IBaseController<Actividad, int>
    {
    }

    /// <summary>
    ///  Controlador de actividad. Hereda del controlador base e implementa la interfaz de controlador de actividad.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class ActividadController : BaseController, IActividadController
    {
        private const int FilasPorPagina = 10; // Número de filas por página 
        private readonly IActividadService srvActividad; // Interfaz del servicio de datos de actividad

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public ActividadController()
        //{
        //    this.srvActividad = new ActividadService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvActividad">Interfaz de servicio de actividad</param>
        /// <returns></returns>
        public ActividadController(IActividadService srvActividad)
        {
            this.srvActividad = srvActividad;
        }

        #region IActividadController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de actividad
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de actividad</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdActividad", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var actividades = new ActividadViewModel
                                  {
                                      Actividades =
                                          srvActividad.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir,
                                                                     textoBuscar),
                                      FilasTotal = srvActividad.Contar(),
                                      FilasPorPagina = FilasPorPagina,
                                      TextoBusqueda = textoBuscar
                                  };

            // Devuelve una vista con los datos
            return View(actividades);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva actividad
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la actividad por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            return View(new Actividad());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de una nueva actividad
        /// </summary>
        /// <param name="entity">Actividad actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(Actividad entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvActividad.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una actividad
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la actividad</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Actividad actividad = srvActividad.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (actividad == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(actividad);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la actividad
        /// </summary>
        /// <param name="entity">Actividad actual</param>
        [HttpPost]
        public ActionResult Editar(Actividad entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvActividad.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una actividad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la actividad, sino muestra una vista con el error
            if (!srvActividad.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}