﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de contacto de entidad. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IContactoController : IBaseController<Contacto, int>
    {
    }

    /// <summary>
    /// Controlador de contacto. Hereda del controlador base e implementa la interfaz de controlador de contacto.
    /// </summary>
    [Authorize]
    public class ContactoController : BaseController, IContactoController
    {
        private const int FilasPorPagina = 10; // Número de filas por página
        private readonly IContactoService srvContacto; // Interfaz del servicio de datos de contacto
        private readonly IEntidadService srvEntidad;   // Interfaz del servicio de datos de entidades       

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public ContactoController()
        //{
        //    this.srvContacto = new ContactoService();
        //    this.srvEntidad = new EntidadService().ObtenerTodos();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvContacto">Interfaz de servicio de contacto</param>
        /// <param name="srvEntidad">Interfaz de servicio de entidad</param>
        /// <returns></returns>
        public ContactoController(IContactoService srvContacto, IEntidadService srvEntidad)
        {
            this.srvContacto = srvContacto;
            this.srvEntidad = srvEntidad;
        }

        #region IContactoController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del contacto
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del contacto</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdContacto", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var contactos = new ContactoViewModel
                               {   Contactos = srvContacto.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvContacto.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(contactos);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo contacto
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del contacto por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea un modelo de la vista para editar el nuevo contacto
            var contactos = new ContactoViewModelEdit
                               {
                                   Contacto = new Contacto(),
                                   Entidades = srvEntidad.ObtenerTodos()
                               };

            // Se definen las propiedades por defecto
            contactos.Contacto.IdEntidad = 1;       // Por defecto un nuevo contacto es de la primera entidad

            // Devuelve una vista para añadir un nuevo contacto
            return View(contactos);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo contacto
        /// </summary>
        /// <param name="entity">Contacto actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Contacto")] Contacto entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var contactos = new ContactoViewModelEdit
                                   {
                                       Contacto = entity,
                                       Entidades = srvEntidad.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(contactos);
            }

            // Actualiza los datos
            if (!srvContacto.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un contacto
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del contacto</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Contacto contacto = srvContacto.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (contacto == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var contactos = new ContactoViewModelEdit
                               {
                                   Contacto = contacto,
                                   Entidades = srvEntidad.ObtenerTodos()
                               };

            // Devuelve una vista para editar
            return View(contactos);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del contacto
        /// </summary>
        /// <param name="entity">Contacto actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Contacto")] Contacto entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var contactos = new ContactoViewModelEdit
                                   {
                                       Contacto = entity,
                                       Entidades = srvEntidad.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(contactos);
            }

            // Actualiza los datos
            if (!srvContacto.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un contacto a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el contacto, sino muestra una vista con el error
            if (id <= 1 || !srvContacto.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

    }
}