﻿using System.Linq;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de delegación. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IDelegacionController : IBaseController<Delegacion, int>
    {
    }

    /// <summary>
    /// Controlador de delegación. Hereda del controlador base e implementa la interfaz de controlador de delegación.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class DelegacionController : BaseController, IDelegacionController
    {
        private const int FilasPorPagina = 10; // Número de filas por página
        private readonly IDelegacionService srvDelegacion; // Interfaz del servicio de datos de delegaciones
        private readonly IOrganizacionService srvOrganizacion; // Interfaz del servicio de datos de organizaciones
        private readonly IPoblacionService srvPoblacion; // Interfaz del servicio de datos de poblaciones

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public DelegacionController()
        //{
        //    this.srvDelegacion = new DelegacionService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvDelegacion">Interfaz de servicio de delegacion</param>
        /// <param name="srvOrganizacion">Interfaz de servicio de organización</param>
        /// <param name="srvPoblacion">Interfaz de servicio de población</param>
        /// <returns></returns>
        public DelegacionController(IDelegacionService srvDelegacion, IOrganizacionService srvOrganizacion,
                                    IPoblacionService srvPoblacion)
        {
            this.srvDelegacion = srvDelegacion;
            this.srvOrganizacion = srvOrganizacion;
            this.srvPoblacion = srvPoblacion;
        }

        #region IDelegacionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de la delegación
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de la delegación</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdDelegacion", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var delegaciones = new DelegacionViewModel
                                   {
                                       Delegaciones = srvDelegacion.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                       FilasTotal = srvDelegacion.Contar(),
                                       FilasPorPagina = FilasPorPagina,
                                       TextoBusqueda = textoBuscar
                                   };

            // Devuelve una vista con los datos
            return View(delegaciones);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva delegacion
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la delegacion por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea el modelo de la vista para editar la nueva delegacion
            var delegacion = new Delegacion();
            var delegaciones = new DelegacionViewModelEdit
                                   {
                                       Delegacion = delegacion,
                                       Organizaciones = srvOrganizacion.ObtenerTodos(),
                                       // Poblaciones = srvPoblacion.ObtenerTodos()
                                       Poblacion = delegacion.Poblacion == null ? "" : delegacion.Poblacion.NombreCompleto 
                                   };

            // Se definen las propiedades por defecto
            delegaciones.Delegacion.IdOrganizacion = 1; // Por defecto la primera organizacion
            delegaciones.Delegacion.IdPoblacion = 1; // Por defecto la primera población

            // Devuelve una vista para añadir una nueva delegacion
            return View(delegaciones);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de la nueva delegacion
        /// </summary>
        /// <param name="entity">Delegación actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Delegacion")] Delegacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var delegaciones = new DelegacionViewModelEdit
                                       {
                                           Delegacion = entity,
                                           Organizaciones = srvOrganizacion.ObtenerTodos(),
                                           //Poblaciones = srvPoblacion.ObtenerTodos()
                                           Poblacion = entity.Poblacion == null ? "" : entity.Poblacion.NombreCompleto 
                                       };

                // Vuelve a editar el modelo y muestra los errores
                return View(delegaciones);
            }

            // Actualiza los datos
            if (!srvDelegacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una delegación
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la delegación</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Delegacion delegacion = srvDelegacion.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (delegacion == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var delegaciones = new DelegacionViewModelEdit
                                   {
                                       Delegacion = delegacion,
                                       Organizaciones = srvOrganizacion.ObtenerTodos(),
                                       //Poblaciones = srvPoblacion.ObtenerTodos()
                                       Poblacion = delegacion.Poblacion == null ? "" : delegacion.Poblacion.NombreCompleto 
                                   };

            // Devuelve una vista para editar
            return View(delegaciones);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la delegación
        /// </summary>
        /// <param name="entity">Delegación actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Delegacion")] Delegacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var delegaciones = new DelegacionViewModelEdit
                                       {
                                           Delegacion = entity,
                                           Organizaciones = srvOrganizacion.ObtenerTodos(),
                                           //Poblaciones = srvPoblacion.ObtenerTodos()
                                           Poblacion = entity.Poblacion == null ? "" : entity.Poblacion.NombreCompleto 
                                       };

                // Vuelve a editar el modelo y muestra los errores
                return View(delegaciones);
            }

            // Actualiza los datos
            if (!srvDelegacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una delegación a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la delegación, sino muestra una vista con el error
            if (!srvDelegacion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion ObtenerPoblaciones (Get). Obtiene las poblaciones a partir del nombre
        /// </summary>
        /// <param name="q">Texto a buscar</param>
        /// <param name="limit">Limite en el número de lineas</param>
        public ActionResult ObtenerPoblaciones(string q, int limit)
        {
            var poblaciones = srvPoblacion.ObtenerNombre(q).Take(limit);
            var datos = from p in poblaciones
                        select new { Id = p.IdPoblacion, Nombre = p.NombreCompleto };
            return Json(datos, JsonRequestBehavior.AllowGet);
        }

        #endregion
    }
}