﻿using System;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;
using GPAS.Utility; 

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de documento de evento. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IDocumentoController : IBaseController<Documento, int>
    {
    }

    /// <summary>
    /// Controlador de documento. Hereda del controlador base e implementa la interfaz de controlador de documento.
    /// </summary>
    [Authorize]
    [HandleError]
    public class DocumentoController : BaseController, IDocumentoController
    {
        private const int FilasPorPagina = 10; // Número de filas por página
        private readonly IDocumentoService srvDocumento;    // Interfaz del servicio de datos de documento
        private readonly IEventoService srvEvento;          // Interfaz del servicio de datos de evento
        private string rutaDocumentos;                      // Ruta virtual de los documentos

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public DocumentoController()
        //{
        //    this.srvDocumento = new DocumentoService();
        //    this.srvEvento = new EventoService().GetAll();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvDocumento">Interfaz de servicio de documento</param>
        /// <param name="srvEvento">Interfaz de servicio de evento</param>
        /// <returns></returns>
        public DocumentoController(IDocumentoService srvDocumento, IEventoService srvEvento)
        {
            this.srvDocumento = srvDocumento;
            this.srvEvento = srvEvento;

            // Obtiene la ruta del web.config
            this.rutaDocumentos = System.Configuration.ConfigurationManager.AppSettings["rutaDocs"];
        }

        #region IDocumentoController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del documento
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del documento</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdDocumento", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var documentos = new DocumentoViewModel
                               {   Documentos = srvDocumento.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvDocumento.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(documentos);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo documento
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del documento por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea un modelo de la vista para editar el nuevo documento
            var documentos = new DocumentoViewModelEdit
                               {
                                   Documento = new Documento(),
                                   RutaDocumentos = rutaDocumentos, 
                                   Eventos = srvEvento.ObtenerTodos()
                               };

            // Se definen las propiedades por defecto
            documentos.Documento.IdEvento = 1;          // Por defecto un nuevo documento es de el primer evento
            documentos.Documento.Fecha = DateTime.Now;  // Por defecto un nuevo documento es de la fecha actual

            // Devuelve una vista para añadir un nuevo documento
            return View(documentos);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo documento
        /// </summary>
        /// <param name="entity">Documento actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Documento")] Documento entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var documentos = new DocumentoViewModelEdit
                                   {
                                       Documento = entity,
                                       RutaDocumentos = rutaDocumentos, 
                                       Eventos = srvEvento.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(documentos);
            }

            // Actualiza los datos
            if (!srvDocumento.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un documento
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del documento</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Documento documento = srvDocumento.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (documento == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var documentos = new DocumentoViewModelEdit
                               {
                                   Documento = documento,
                                   RutaDocumentos = rutaDocumentos, 
                                   Eventos = srvEvento.ObtenerTodos()
                               };

            // Devuelve una vista para editar
            return View(documentos);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del documento
        /// </summary>
        /// <param name="entity">Documento actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Documento")] Documento entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var documentos = new DocumentoViewModelEdit
                                   {
                                       Documento = entity,
                                       RutaDocumentos = rutaDocumentos, 
                                       Eventos = srvEvento.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(documentos);
            }

            // Actualiza los datos
            if (!srvDocumento.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un documento a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el documento, sino muestra una vista con el error
            if (id <= 1 || !srvDocumento.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

        #region IFileController Members

        /// <summary>
        /// Accion Subir (Post). Valida y sube un documento al servidor
        /// </summary>
        /// <param name="archivo">Documento actual</param>
        /// <returns>Si es válido, sube el documento, sino vuelve la vista 'Subir' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Subir(HttpPostedFileBase archivo)
        {           
            string iddocumento = HttpContext.Request.Form["IdDocumento"];

            // Comprueba la longitud del fichero
            if (archivo != null && archivo.ContentLength > 0)
            {
                // Obtiene el nombre y la ruta del fichero
                var documento = (string.IsNullOrEmpty(iddocumento) ? "" : iddocumento + "_") + Path.GetFileName(archivo.FileName);
                var rutaFichero = Path.Combine(Server.MapPath(rutaDocumentos), documento);

                // Comprueba si el archivo existe (esta comprobación posiblemente pertenezca el Modelo).
                if (System.IO.File.Exists(rutaFichero))
                {
                    ModelState.AddModelError("Nombre", string.Format("El fichero <strong>{0}</strong> ya existe.",
                                             Path.GetFileName(archivo.FileName)));
                }
                else
                {
                    // Guarda el archivo en el servidor
                    archivo.SaveAs(rutaFichero);

                    //return Json(new { status = "valido", message = string.Format("El fichero <strong>{0}</strong> se ha subido correctamente.<br /><br/>",
                    //            Path.GetFileName(archivo.FileName)) });
                }
            }
            else
            {
                ModelState.AddModelError("Nombre", string.Format("El fichero <strong>{0}</strong> no es válido.",
                                         Path.GetFileName(archivo.FileName)));
            }

            // Vuelve a la ruta anterior
            return Redirect(HttpContext.Request.UrlReferrer.AbsolutePath);
        }

        /// <summary>
        /// Accion Descargar (Get). Descarga un documento del servidor
        /// </summary>
        /// <returns>Devuelve el archivo con el documento</returns>
        [HttpGet]
        public ActionResult Descargar(string fichero)
        {
            // Comprueba si el archivo existe (esta comprobación posiblemente pertenezca el Modelo).
            if (System.IO.File.Exists(Server.MapPath(rutaDocumentos) + "\\" + fichero))
            {
                // Descarga el documento, eliminando la identificacion
                return new DownloadResult
                            {
                                VirtualPath = rutaDocumentos + "/" + fichero,
                                FileDownloadName = fichero.Substring(fichero.IndexOf("_") + 1)
                            };
            }
            else
            {
                ModelState.AddModelError("Nombre", string.Format("El fichero <strong>{0}</strong> no se encuentra.", fichero));

                // Vuelve a la ruta anterior
                return Redirect(HttpContext.Request.UrlReferrer.AbsolutePath);
            }
        }

        #endregion

    }
}