﻿using System.Linq;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

//using GPAS.Filters;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de entidad de colaboración. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IEntidadController : IBaseController<Entidad, int>
    {
    }

    /// <summary>
    /// Controlador de  entidad de colaboración. Hereda del controlador base e implementa la interfaz de controlador de entidad.
    /// </summary>
    [Authorize]
    public class EntidadController : BaseController, IEntidadController
    {
        public const int FilasPorPagina = 10;                       // Número de filas por página
        private readonly IEntidadService srvEntidad;                // Interfaz del servicio de datos de entidades
        private readonly ITipoEntidadService srvTipoEntidad;        // Interfaz del servicio de datos de tipos de entidades
        private readonly IPoblacionService srvPoblacion;            // Interfaz del servicio de datos de poblaciones
        private readonly IContactoService srvContacto;              // Interfaz del servicio de datos de entidades
        private readonly ISeguimientoService srvSeguimiento;        // Interfaz del servicio de datos de tipos de entidades
        private readonly IParticipacionService srvParticipacion;    // Interfaz del servicio de datos de poblaciones

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public EntidadController()
        //{
        //    this.srvEntidad = new EntidadService();
        //    this.srvTipoEntidad = new TipoEntidadService();
        //    this.srvPoblacion = new PoblacionService();
        //    this.srvContacto = new ContactoService();
        //    this.srvSeguimiento = new SeguimientoService();
        //    this.srvParticipacion = new ParticipacionService();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvEntidad">Interfaz de servicio de entidad</param>
        /// <param name="srvTipoEntidad">Interfaz de servicio de tipo de entidad</param>
        /// <param name="srvPoblacion">Interfaz de servicio de población</param>
        /// <param name="srvContacto">Interfaz de servicio de entidad</param>
        /// <param name="srvSeguimiento">Interfaz de servicio de tipo de entidad</param>
        /// <param name="srvParticipacion">Interfaz de servicio de población</param>
        /// <returns></returns>
        public EntidadController(IEntidadService srvEntidad, ITipoEntidadService srvTipoEntidad, IPoblacionService srvPoblacion,
                                 IContactoService srvContacto, ISeguimientoService srvSeguimiento, IParticipacionService srvParticipacion)
        {
            this.srvEntidad = srvEntidad;
            this.srvTipoEntidad = srvTipoEntidad;
            this.srvPoblacion = srvPoblacion;
            this.srvContacto = srvContacto;
            this.srvSeguimiento = srvSeguimiento;
            this.srvParticipacion = srvParticipacion;
        }

        #region IEntidadController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de la entidad
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de la entidad</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdEntidad", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var entidades = new EntidadViewModel
                                {
                                    Entidades = srvEntidad.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                    FilasTotal = srvEntidad.Contar(),
                                    FilasPorPagina = FilasPorPagina,
                                    TextoBusqueda = textoBuscar
                                };

            // Devuelve una vista con los datos
            return View(entidades);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva entidad
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la entidad por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea una nueva entidad
            var entidad = new Entidad();

            // Crea el modelo de la vista para editar la nueva entidad
            var entidades = new EntidadViewModelEdit
                                {
                                    Entidad = entidad,
                                    TiposEntidades = srvTipoEntidad.ObtenerTodos(),
                                    Poblacion = entidad.Poblacion == null ? "" : entidad.Poblacion.NombreCompleto,
                                    Contactos = srvContacto.ObtenerEntidad(entidad.IdEntidad),
                                    Seguimientos = srvSeguimiento.ObtenerEntidad(entidad.IdEntidad),
                                    Participaciones = srvParticipacion.ObtenerEntidad(entidad.IdEntidad)
                                };

            // Se definen las propiedades por defecto
            entidades.Entidad.IdTipoEntidad = 1;        // Por defecto el primero tipo de entidad
            entidades.Entidad.IdPoblacion = 1;          // Por defecto la primera población

            // Devuelve una vista para añadir una nueva entidad
            return View(entidades);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de la nueva entidad
        /// </summary>
        /// <param name="entity">Entidad actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Entidad")] Entidad entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var entidades = new EntidadViewModelEdit
                                    {
                                        Entidad = entity,
                                        TiposEntidades = srvTipoEntidad.ObtenerTodos(),
                                        Poblacion = entity.Poblacion == null ? "" : entity.Poblacion.NombreCompleto,
                                        Contactos = srvContacto.ObtenerEntidad(entity.IdEntidad),
                                        Seguimientos = srvSeguimiento.ObtenerEntidad(entity.IdEntidad),
                                        Participaciones = srvParticipacion.ObtenerEntidad(entity.IdEntidad) 
                                    };
                // Vuelve a editar el modelo y muestra los errores
                return View(entidades);
            }

            // Actualiza los datos
            if (!srvEntidad.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una entidad
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la entidad</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Entidad entidad = srvEntidad.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (entidad == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var entidades = new EntidadViewModelEdit
                                {
                                    Entidad = entidad,
                                    TiposEntidades = srvTipoEntidad.ObtenerTodos(),
                                    Poblacion = entidad.Poblacion == null ? "" : entidad.Poblacion.NombreCompleto,
                                    Contactos = srvContacto.ObtenerEntidad(entidad.IdEntidad),
                                    Seguimientos = srvSeguimiento.ObtenerEntidad(entidad.IdEntidad),
                                    Participaciones = srvParticipacion.ObtenerEntidad(entidad.IdEntidad)
                                };

            // Devuelve una vista para editar
            return View(entidades);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la entidad
        /// </summary>
        /// <param name="entity">Entidad actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Entidad")] Entidad entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var entidades = new EntidadViewModelEdit
                                    {
                                        Entidad = entity,
                                        TiposEntidades = srvTipoEntidad.ObtenerTodos(),
                                        Poblacion = entity.Poblacion == null ? "" : entity.Poblacion.NombreCompleto,
                                        Contactos = srvContacto.ObtenerEntidad(entity.IdEntidad),
                                        Seguimientos = srvSeguimiento.ObtenerEntidad(entity.IdEntidad),
                                        Participaciones = srvParticipacion.ObtenerEntidad(entity.IdEntidad)
                                    };

                // Vuelve a editar el modelo y muestra los errores
                return View(entidades);
            }

            // Actualiza los datos
            if (!srvEntidad.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una entidad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la entidad, sino muestra una vista con el error
            if (!srvEntidad.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion ObtenerPoblaciones (Get). Obtiene las poblaciones a partir del nombre
        /// </summary>
        /// <param name="q">Texto a buscar</param>
        /// <param name="limit">Limite en el número de lineas</param>
        public ActionResult ObtenerPoblaciones(string q, int limit)
        {
            var poblaciones = srvPoblacion.ObtenerNombre(q).Take(limit);
            var datos = from p in poblaciones
                        select new { Id = p.IdPoblacion, Nombre = p.NombreCompleto };
            return Json(datos, JsonRequestBehavior.AllowGet);
        }

        #endregion
    }
}