﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    /// Interfaz de controlador de estado de entidad. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IEstadoController : IBaseController<Estado, int>
    {
    }

    /// <summary>
    /// Controlador de estado de entidad. Hereda del controlador base e implementa la interfaz de controlador de estado.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class EstadoController : BaseController, IEstadoController
    {
        public const int FilasPorPagina = 10; // Número de filas por página 
        private readonly IEstadoService srvEstado; // Interfaz del servicio de datos de estado

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public EstadoController()
        //{
        //    this.srvEstado = new EstadoService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvEstado">Interfaz de servicio de estado de entidad</param>
        /// <returns></returns>
        public EstadoController(IEstadoService srvEstado)
        {
            this.srvEstado = srvEstado;
        }

        #region IEstadoController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de estado de entidad
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de estado de entidad</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdEstado", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var estados = new EstadoViewModel
                              {
                                  Estados =
                                      srvEstado.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                  FilasTotal = srvEstado.Contar(),
                                  FilasPorPagina = FilasPorPagina,
                                  TextoBusqueda = textoBuscar
                              };

            // Devuelve una vista con los datos
            return View(estados);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo estado de entidad
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de el estado por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir un nuevo estado de entidad
            return View(new Estado());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de un nuevo estado de entidad
        /// </summary>
        /// <param name="entity">Estado actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(Estado entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvEstado.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un estado de entidad
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del estado de entidad</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Estado estado = srvEstado.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (estado == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(estado);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del estado de entidad
        /// </summary>
        /// <param name="entity">Estado actual</param>
        [HttpPost]
        public ActionResult Editar(Estado entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvEstado.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un estado de entidad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el estado de entidad, sino muestra una vista con el error
            if (!srvEstado.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}