﻿using System.Linq;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;
using System.Collections.Generic;
using System;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de evento de acción. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IEventoController : IBaseController<Evento, int>
    {
    }

    /// <summary>
    /// Controlador de evento. Hereda del controlador base e implementa la interfaz de controlador de evento.
    /// </summary>
    [Authorize]
    public class EventoController : BaseController, IEventoController
    {
        private const int FilasPorPagina = 7;                     // Número de filas por página
        private readonly IEventoService srvEvento;                // Interfaz del servicio de datos de eventos
        private readonly IAccionService srvAccion;                // Interfaz del servicio de datos de acciones
        private readonly IPoblacionService srvPoblacion;          // Interfaz del servicio de datos de poblaciones
        private readonly IDocumentoService srvDocumento;          // Interfaz del servicio de datos de documentos
        private readonly IFotoService srvFoto;                    // Interfaz del servicio de datos de fotos
        private readonly IParticipacionService srvParticipacion;  // Interfaz del servicio de datos de participaciones

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public EventoController()
        //{
        //    this.srvEvento = new EventoService();
        //    this.srvAccion = new AccionService().GetAll();
        //    this.srvPoblacion = new PoblacionService().GetAll();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvEvento">Interfaz de servicio de evento</param>
        /// <param name="srvAccion">Interfaz de servicio de acción</param>
        /// <param name="srvPoblacion">Interfaz de servicio de población</param>
        /// <param name="srvDocumento">Interfaz de servicio de documento</param>
        /// <param name="srvFoto">Interfaz de servicio de foto</param>
        /// <param name="srvParticipacion">Interfaz de servicio de participación</param>
        /// <returns></returns>
        public EventoController(IEventoService srvEvento, IAccionService srvAccion, IPoblacionService srvPoblacion,
                                IDocumentoService srvDocumento, IFotoService srvFoto, IParticipacionService srvParticipacion)
        {
            this.srvEvento = srvEvento;
            this.srvAccion = srvAccion;
            this.srvPoblacion = srvPoblacion;
            this.srvDocumento = srvDocumento;
            this.srvFoto = srvFoto;
            this.srvParticipacion = srvParticipacion;
        }

        #region IEventoController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del evento
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del evento</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdEvento", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var eventos = new EventoViewModel
                               {   Eventos = srvEvento.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvEvento.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(eventos);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo evento
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del evento por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {           
            // Crea un modelo de la vista para editar el nuevo evento
            var evento = new Evento();
            var eventos = new EventoViewModelEdit
                               {
                                   Evento = evento,
                                   Acciones = srvAccion.ObtenerTodos(),
                                   Poblacion = evento.Poblacion == null ? "" : evento.Poblacion.NombreCompleto,
                                   Documentos = srvDocumento.ObtenerEvento(evento.IdEvento),
                                   Fotos = srvFoto.ObtenerEvento(evento.IdEvento),
                                   Participaciones = srvParticipacion.ObtenerEvento(evento.IdEvento)
                               };

            // Se definen las propiedades por defecto
            eventos.Evento.IdAccion = 1;                        // Por defecto un nuevo evento es de la primera acción
            eventos.Evento.IdPoblacion = 1;                     // Por defecto un nuevo evento es de la primera población
            eventos.Evento.FechaInicio = DateTime.Now.Date;          // Por defecto un nuevo evento inicia en la fecha actual
            eventos.Evento.FechaFin = DateTime.Now.Date.AddDays(30); // Por defecto un nuevo evento finaliza en un mes

            // Devuelve una vista para añadir un nuevo evento
            return View(eventos);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo evento
        /// </summary>
        /// <param name="entity">Evento actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Evento")] Evento entity)
        {
                // Si el modelo no es válido
                if (!ModelState.IsValid)
                {
                    // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                    var eventos = new EventoViewModelEdit
                                       {
                                           Evento = entity,
                                           Acciones = srvAccion.ObtenerTodos(),
                                           Poblacion = entity.Poblacion == null ? "" : entity.Poblacion.NombreCompleto,
                                           Documentos = srvDocumento.ObtenerEvento(entity.IdEvento),
                                           Fotos = srvFoto.ObtenerEvento(entity.IdEvento),
                                           Participaciones = srvParticipacion.ObtenerEvento(entity.IdEvento)
                                       };

                    // Vuelve a editar el modelo y muestra los errores
                    return View(eventos);
                }

                // Actualiza los datos
                if (!srvEvento.Guardar(entity))                  
                   return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un evento
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del evento</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Evento evento = srvEvento.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (evento == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var eventos = new EventoViewModelEdit
                               {
                                   Evento = evento,
                                   Acciones = srvAccion.ObtenerTodos(),
                                   Poblacion = evento.Poblacion == null ? "" : evento.Poblacion.NombreCompleto,
                                   Documentos = srvDocumento.ObtenerEvento(evento.IdEvento),
                                   Fotos = srvFoto.ObtenerEvento(evento.IdEvento),
                                   Participaciones = srvParticipacion.ObtenerEvento(evento.IdEvento)
                               };

            // Devuelve una vista para editar
            return View(eventos);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del evento
        /// </summary>
        /// <param name="entity">Evento actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Evento")] Evento entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var eventos = new EventoViewModelEdit
                                    {
                                        Evento = entity,
                                        Acciones = srvAccion.ObtenerTodos(),
                                        Poblacion = entity.Poblacion == null ? "" : entity.Poblacion.NombreCompleto,
                                        Documentos = srvDocumento.ObtenerEvento(entity.IdEvento),
                                        Fotos = srvFoto.ObtenerEvento(entity.IdEvento),
                                        Participaciones = srvParticipacion.ObtenerEvento(entity.IdEvento)
                                    };

                // Vuelve a editar el modelo y muestra los errores
                return View(eventos);
            }

            // Actualiza los datos
            if (!srvEvento.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un evento a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el evento, sino muestra una vista con el error
            if (id <= 1 || !srvEvento.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

        /// <summary>
        /// Accion ObtenerPoblaciones (Get). Obtiene las poblaciones a partir del nombre
        /// </summary>
        /// <param name="q">Texto a buscar</param>
        /// <param name="limit">Limite en el número de lineas</param>
        public ActionResult ObtenerPoblaciones(string q, int limit)
        {
            var poblaciones = srvPoblacion.ObtenerNombre(q).Take(limit);
            var datos = from p in poblaciones
                        select new {Id = p.IdPoblacion, Nombre = p.NombreCompleto};
            return Json(datos, JsonRequestBehavior.AllowGet);
        }

        /// <summary>
        /// Accion GetMarkersAsync (Post). Obtiene los eventos en formato GoogleMarker
        /// </summary>
        [HttpPost]
        public ActionResult GetMarkersAsync()
        {
            var googleMarkers = new List<GoogleMarker>();

            // Obtenemos los eventos
            var eventos = srvEvento.ObtenerTodos();

            foreach (Evento evento in eventos)
            {
                // Rellenamos la lista de eventos localizados
                googleMarkers.Add(evento.ToGoogleMarker());
            }

            return Json(googleMarkers);
        }

    }
}