﻿using System;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;
using GPAS.Utility; 

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de foto de evento. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IFotoController : IBaseController<Foto, int>
    {
    }

    /// <summary>
    /// Controlador de foto. Hereda del controlador base e implementa la interfaz de controlador de foto.
    /// </summary>
    [Authorize]
    [HandleError]
    public class FotoController : BaseController, IFotoController, IFileController
    {
        private const int FilasPorPagina = 10; // Número de filas por página
        private readonly IFotoService srvFoto;              // Interfaz del servicio de datos de foto
        private readonly IEventoService srvEvento;          // Interfaz del servicio de datos de evento
        private string rutaFotos;                           // Ruta virtual de las fotos

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public FotoController()
        //{
        //    this.srvFoto = new FotoService();
        //    this.srvEvento = new EventoService().GetAll();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvFoto">Interfaz de servicio de foto</param>
        /// <param name="srvEvento">Interfaz de servicio de evento</param>
        /// <returns></returns>
        public FotoController(IFotoService srvFoto, IEventoService srvEvento)
        {
            this.srvFoto = srvFoto;
            this.srvEvento = srvEvento;

            // Obtiene la ruta del web.config
            this.rutaFotos = System.Configuration.ConfigurationManager.AppSettings["rutaFotos"];
        }

        #region IFotoController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del foto
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del foto</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdFoto", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var fotos = new FotoViewModel
                               {   Fotos = srvFoto.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvFoto.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(fotos);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo foto
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del foto por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea un modelo de la vista para editar el nuevo foto
            var fotos = new FotoViewModelEdit
                               {
                                   Foto = new Foto(),
                                   RutaFotos = rutaFotos, 
                                   Eventos = srvEvento.ObtenerTodos()
                               };

            // Se definen las propiedades por defecto
            fotos.Foto.IdEvento = 1;                    // Por defecto una nueva foto es de el primer evento
            fotos.Foto.Fecha = DateTime.Now;            // Por defecto una nueva foto es de la fecha actual

            // Devuelve una vista para añadir un nuevo foto
            return View(fotos);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo foto
        /// </summary>
        /// <param name="entity">Foto actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Foto")] Foto entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var fotos = new FotoViewModelEdit
                                   {
                                       Foto = entity,
                                       RutaFotos = rutaFotos, 
                                       Eventos = srvEvento.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(fotos);
            }

            // Actualiza los datos
            if (!srvFoto.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un foto
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del foto</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Foto foto = srvFoto.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (foto == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var fotos = new FotoViewModelEdit
                               {
                                   Foto = foto,
                                   RutaFotos = rutaFotos, 
                                   Eventos = srvEvento.ObtenerTodos()
                               };

            // Devuelve una vista para editar
            return View(fotos);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del foto
        /// </summary>
        /// <param name="entity">Foto actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Foto")] Foto entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var fotos = new FotoViewModelEdit 
                                    { 
                                        Foto = entity, 
                                        RutaFotos = rutaFotos, 
                                        Eventos = srvEvento.ObtenerTodos() 
                                    };

                // Vuelve a editar el modelo y muestra los errores
                return View(fotos);
            }

            // Actualiza los datos
            if (!srvFoto.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un foto a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el foto, sino muestra una vista con el error
            if (id <= 1 || !srvFoto.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

        #region IFileController Members

        /// <summary>
        /// Accion Subir (Post). Valida y sube una foto al servidor
        /// </summary>
        /// <param name="archivo">Foto actual</param>
        /// <returns>Si es válido, sube la foto, sino vuelve la vista 'Subir' mostrando los errores</returns>
        public ActionResult Subir(HttpPostedFileBase archivo)
        {
            string idfoto = HttpContext.Request.Form["IdFoto"];

            // Comprueba la longitud del fichero
            if (archivo != null && archivo.ContentLength > 0)
            {
                // Obtiene el nombre y la ruta del fichero
                var foto = (string.IsNullOrEmpty(idfoto) ? "" : idfoto + "_") + Path.GetFileName(archivo.FileName);
                var rutaFichero = Path.Combine(Server.MapPath(rutaFotos), foto);

                // Comprueba si el archivo existe (esta comprobación posiblemente pertenezca el Modelo).
                if (System.IO.File.Exists(rutaFichero))
                {
                    ModelState.AddModelError("Nombre",  string.Format("El fichero <strong>{0}</strong> ya existe.", 
                                             Path.GetFileName(archivo.FileName)));
                }
                else
                {
                    // Guarda el archivo en el servidor
                    archivo.SaveAs(rutaFichero);

                    //return Json(new { status = "valido", message = string.Format("El fichero <strong>{0}</strong> se ha subido correctamente.<br /><br/>",
                    //            Path.GetFileName(archivo.FileName)) });
                }
            }
            else
            {
                ModelState.AddModelError("Nombre", string.Format("El fichero <strong>{0}</strong> no es válido.",
                                         Path.GetFileName(archivo.FileName)));
            }

            // Vuelve a la ruta anterior
            return Redirect(HttpContext.Request.UrlReferrer.AbsolutePath);
        }

        /// <summary>
        /// Accion Descargar (Get). Descarga una foto del servidor
        /// </summary>
        /// <returns>Devuelve el archivo con la foto</returns>
        [HttpGet]
        public ActionResult Descargar(string fichero)
        {
            // Comprueba si el archivo existe (esta comprobación posiblemente pertenezca el Modelo).
            if (System.IO.File.Exists(Server.MapPath(rutaFotos) + "\\" + fichero))
            {
                // Descarga la foto, eliminando la identificacion
                return new DownloadResult 
                            { 
                                VirtualPath = rutaFotos + "/" + fichero, 
                                FileDownloadName = fichero.Substring(fichero.IndexOf("_") + 1) 
                            };
            }
            else
            {
                ModelState.AddModelError("Nombre", string.Format("El fichero <strong>{0}</strong> no se encuentra.", fichero));

                // Vuelve a la ruta anterior
                return Redirect(HttpContext.Request.UrlReferrer.AbsolutePath);
            }
        }

        #endregion
    }
}