﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    /// Interfaz de controlador de medio de entidad. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IMedioController : IBaseController<Medio, int>
    {
    }

    /// <summary>
    /// Controlador de medio de entidad. Hereda del controlador base e implementa la interfaz de controlador de medio.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class MedioController : BaseController, IMedioController
    {
        private const int FilasPorPagina = 10; // Número de filas por página 
        private readonly IMedioService srvMedio; // Interfaz del servicio de datos de medio

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public MedioController() 
        //{
        //    this.srvMedio = new MedioService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvMedio">Interfaz de servicio de medio de entidad</param>
        /// <returns></returns>
        public MedioController(IMedioService srvMedio)
        {
            this.srvMedio = srvMedio;
        }

        #region IMedioController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de medio de entidad
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de medio de entidad</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdMedio", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var medios = new MedioViewModel
                             {
                                 Medios = srvMedio.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                 FilasTotal = srvMedio.Contar(),
                                 FilasPorPagina = FilasPorPagina,
                                 TextoBusqueda = textoBuscar
                             };

            // Devuelve una vista con los datos
            return View(medios);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo medio de entidad
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de el medio por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir un nuevo medio de entidad
            return View(new Medio());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de un nuevo medio de entidad
        /// </summary>
        /// <param name="entity">Medio actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(Medio entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvMedio.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un medio de entidad
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del medio de entidad</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Medio medio = srvMedio.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (medio == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(medio);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del medio de entidad
        /// </summary>
        /// <param name="entity">Medio actual</param>
        [HttpPost]
        public ActionResult Editar(Medio entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvMedio.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un medio de entidad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el medio de entidad, sino muestra una vista con el error
            if (!srvMedio.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}