﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de organización. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IOrganizacionController : IBaseController<Organizacion, int>
    {
    }

    /// <summary>
    /// Controlador de organización. Hereda del controlador base e implementa la interfaz de controlador de organización.
    /// </summary>
    public class OrganizacionController : BaseController, IOrganizacionController
    {
        public const int FilasPorPagina = 10;                           // Número de filas por página
        private readonly IOrganizacionService srvOrganizacion;          // Interfaz del servicio de datos de organizaciones
        private readonly ITipoOrganizacionService srvTipoOrganizacion;  // Interfaz del servicio de datos de tipos de organizaciones

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public OrganizacionController()
        //{
        //    this.srvOrganizacion = new OrganizacionService();
        //    this.srvTipoOrganizacion = new TipoOrganizacionService();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvOrganizacion">Interfaz de servicio de organización</param>
        /// <param name="srvTipoOrganizacion">Interfaz de servicio de tipo de organización</param>
        /// <returns></returns>
        public OrganizacionController(IOrganizacionService srvOrganizacion, ITipoOrganizacionService srvTipoOrganizacion)
        {
            this.srvOrganizacion = srvOrganizacion;
            this.srvTipoOrganizacion = srvTipoOrganizacion;
        }

        #region IOrganizacionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de la organización
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de la organización</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdOrganizacion", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var orgs = new OrganizacionViewModel
                           {
                               Organizaciones = srvOrganizacion.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                               FilasTotal = srvOrganizacion.Contar(),
                               FilasPorPagina = FilasPorPagina,
                               TextoBusqueda = textoBuscar
                           };

            // Devuelve una vista con los datos
            return View(orgs);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva organización
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la organización por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea el modelo de la vista para editar la nueva organización
            var organizaciones = new OrganizacionViewModelEdit
                                     {
                                         Organizacion = new Organizacion(),
                                         TiposOrganizaciones = srvTipoOrganizacion.ObtenerTodos()
                                     };

            // Se definen las propiedades por defecto
            organizaciones.Organizacion.IdTipoOrganizacion = 1; // Por defecto el primer tipo de organizacion

            // Devuelve una vista para añadir una nueva organización
            return View(organizaciones);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de la nueva organización
        /// </summary>
        /// <param name="entity">Organización actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Organizacion")] Organizacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var organizaciones = new OrganizacionViewModelEdit
                                         {
                                             Organizacion = entity,
                                             TiposOrganizaciones = srvTipoOrganizacion.ObtenerTodos()
                                         };

                // Vuelve a editar el modelo y muestra los errores
                return View(organizaciones);
            }

            // Actualiza los datos
            if (!srvOrganizacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una organización
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la organización</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Organizacion organizacion = srvOrganizacion.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (organizacion == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var organizaciones = new OrganizacionViewModelEdit
                                     {
                                         Organizacion = organizacion,
                                         TiposOrganizaciones = srvTipoOrganizacion.ObtenerTodos()
                                     };

            // Devuelve una vista para editar
            return View(organizaciones);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la organización
        /// </summary>
        /// <param name="entity">Organizacion actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns> 
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Organizacion")] Organizacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var organizaciones = new OrganizacionViewModelEdit
                                         {
                                             Organizacion = entity,
                                             TiposOrganizaciones = srvTipoOrganizacion.ObtenerTodos()
                                         };

                // Vuelve a editar el modelo y muestra los errores
                return View(organizaciones);
            }

            // Actualiza los datos
            if (!srvOrganizacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una organización a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la organización, sino muestra una vista con el error
            if (!srvOrganizacion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}