﻿using System;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de participación de evento. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IParticipacionController : IBaseController<Participacion, int>
    {
    }

    /// <summary>
    /// Controlador de participación. Hereda del controlador base e implementa la interfaz de controlador de participación.
    /// </summary>
    [Authorize]
    public class ParticipacionController : BaseController, IParticipacionController
    {
        private const int FilasPorPagina = 10; // Número de filas por página
        private readonly IParticipacionService srvParticipacion;            // Interfaz del servicio de datos de participaciones
        private readonly IEntidadService srvEntidad;                        // Interfaz del servicio de datos de entidades       
        private readonly IEventoService srvEvento;                          // Interfaz del servicio de datos de eventos
        private readonly ITipoParticipacionService srvTipoParticipacion;    // Interfaz del servicio de datos de tipos de participación

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public ParticipacionController()
        //{
        //    this.srvParticipacion = new ParticipacionService();
        //    this.srvEntidad = new EntidadService().ObtenerTodos();
        //    this.srvEvento = new EventoService().ObtenerTodos();
        //    this.srvTipoParticipacion = new TipoParticipacionService().ObtenerTodos();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvParticipacion">Interfaz de servicio de participación</param>
        /// <param name="srvEntidad">Interfaz de servicio de entidad</param>
        /// <param name="srvEvento">Interfaz de servicio de evento</param>
        /// <param name="srvTipoParticipacion">Interfaz de servicio de tipo de participación</param>
        /// <returns></returns>
        public ParticipacionController(IParticipacionService srvParticipacion, IEntidadService srvEntidad,
                                       IEventoService srvEvento, ITipoParticipacionService srvTipoParticipacion)
        {
            this.srvParticipacion = srvParticipacion;
            this.srvEntidad = srvEntidad;
            this.srvEvento = srvEvento;
            this.srvTipoParticipacion = srvTipoParticipacion;
        }

        #region IParticipacionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de la participación
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de la participación</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdParticipacion", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var participaciones = new ParticipacionViewModel
                               {   Participaciones = srvParticipacion.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvParticipacion.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(participaciones);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva participación
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la participación por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea un modelo de la vista para editar la nueva participación
            var participaciones = new ParticipacionViewModelEdit
                               {
                                   Participacion = new Participacion(),
                                   Entidades = srvEntidad.ObtenerTodos(),
                                   Eventos = srvEvento.ObtenerTodos(),
                                   TiposParticipaciones = srvTipoParticipacion.ObtenerTodos() 
                               };

            // Se definen las propiedades por defecto
            participaciones.Participacion.IdEntidad = 1;                // Por defecto una nueva participación es de la primera entidad
            participaciones.Participacion.IdEvento = 1;                 // Por defecto una nueva participación es de el primer evento
            participaciones.Participacion.IdTipoParticipacion = 1;      // Por defecto una nueva participación es de el primer tipo de participación
            participaciones.Participacion.FechaInicio = DateTime.Now;           // Por defecto una nueva participación inicia en la fecha actual
            participaciones.Participacion.FechaFin = DateTime.Now.AddDays(30);  // Por defecto una nueva participación finaliza en un mes

            // Devuelve una vista para añadir una nueva participación
            return View(participaciones);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de la nueva participación
        /// </summary>
        /// <param name="entity">Participación actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Participacion")] Participacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var participaciones = new ParticipacionViewModelEdit
                                   {
                                       Participacion = entity,
                                       Entidades = srvEntidad.ObtenerTodos(),
                                       Eventos = srvEvento.ObtenerTodos(),
                                       TiposParticipaciones = srvTipoParticipacion.ObtenerTodos() 
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(participaciones);
            }

            // Actualiza los datos
            if (!srvParticipacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una participación
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la participación</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Participacion participacion = srvParticipacion.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (participacion == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var participaciones = new ParticipacionViewModelEdit
                               {
                                   Participacion = participacion,
                                   Entidades = srvEntidad.ObtenerTodos(),
                                   Eventos = srvEvento.ObtenerTodos(),
                                   TiposParticipaciones = srvTipoParticipacion.ObtenerTodos() 
                               };

            // Devuelve una vista para editar
            return View(participaciones);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la participación
        /// </summary>
        /// <param name="entity">Participación actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Participacion")] Participacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var participaciones = new ParticipacionViewModelEdit
                                   {
                                       Participacion = entity,
                                       Entidades = srvEntidad.ObtenerTodos(),
                                       Eventos = srvEvento.ObtenerTodos(),
                                       TiposParticipaciones = srvTipoParticipacion.ObtenerTodos() 
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(participaciones);
            }

            // Actualiza los datos
            if (!srvParticipacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una participación a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la participación, sino muestra una vista con el error
            if (id <= 1 || !srvParticipacion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

    }
}