﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de población. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IPoblacionController : IBaseController<Poblacion, int>
    {
    }

    /// <summary>
    /// Controlador de población. Hereda del controlador base e implementa la interfaz de controlador de población.
    /// </summary>
    [Authorize(Roles = "admin")]
    public class PoblacionController : BaseController, IPoblacionController
    {
        private const int FilasPorPagina = 10;              // Número de filas por página
        private readonly IPoblacionService srvPoblacion;    // Interfaz del servicio de datos de poblaciones
        private readonly IProvinciaService srvProvincia;    // Interfaz del servicio de datos de provincias

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public PoblacionController()
        //{
        //    this.srvPoblacion = new PoblacionService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvPoblacion">Interfaz de servicio de población</param>
        /// <param name="srvProvincia">Interfaz de servicio de provincia</param>
        /// <returns></returns>
        public PoblacionController(IPoblacionService srvPoblacion, IProvinciaService srvProvincia)
        {
            this.srvPoblacion = srvPoblacion;
            this.srvProvincia = srvProvincia;
        }

        #region IPoblacionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de la población
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de la población</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdPoblacion", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var poblaciones = new PoblacionViewModel
                                  {
                                      Poblaciones = srvPoblacion.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir,
                                                                     textoBuscar),
                                      FilasTotal = srvPoblacion.Contar(),
                                      FilasPorPagina = FilasPorPagina,
                                      TextoBusqueda = textoBuscar
                                  };

            // Devuelve una vista con los datos
            return View(poblaciones);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva población
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la población por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea el modelo de la vista para editar la nueva población
            var poblaciones = new PoblacionViewModelEdit
                                  {
                                      Poblacion = new Poblacion(),
                                      Provincias = srvProvincia.ObtenerTodos()
                                  };

            // Se definen las propiedades por defecto
            poblaciones.Poblacion.IdProvincia = 1; // Por defecto la primera provincia

            // Devuelve una vista para añadir una nueva población
            return View(poblaciones);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de la nueva población
        /// </summary>
        /// <param name="entity">Población actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Poblacion")] Poblacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var poblaciones = new PoblacionViewModelEdit
                                      {
                                          Poblacion = entity,
                                          Provincias = srvProvincia.ObtenerTodos()
                                      };

                // Vuelve a editar el modelo y muestra los errores
                return View(poblaciones);
            }

            // Actualiza los datos
            if (!srvPoblacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una población
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la población</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Poblacion poblacion = srvPoblacion.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (poblacion == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var poblaciones = new PoblacionViewModelEdit
                                  {
                                      Poblacion = poblacion,
                                      Provincias = srvProvincia.ObtenerTodos()
                                  };

            // Devuelve una vista para editar
            return View(poblaciones);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la población
        /// </summary>
        /// <param name="entity">població actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Poblacion")] Poblacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var poblaciones = new PoblacionViewModelEdit
                                      {
                                          Poblacion = entity,
                                          Provincias = srvProvincia.ObtenerTodos()
                                      };

                // Vuelve a editar el modelo y muestra los errores
                return View(poblaciones);
            }

            // Actualiza los datos
            if (!srvPoblacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una población a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la población, sino muestra una vista con el error
            if (!srvPoblacion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}