﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de provincia. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IProvinciaController : IBaseController<Provincia, int>
    {
    }

    /// <summary>
    ///  Controlador de provincia. Hereda del controlador base e implementa la interfaz de controlador de provincia.
    /// </summary>
    [Authorize(Roles = "admin")]
    public class ProvinciaController : BaseController, IProvinciaController
    {
        public const int FilasPorPagina = 10; // Número de filas por página 
        private readonly IProvinciaService srvProvincia; // Interfaz del servicio de datos de provincia

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public ProvinciaController()
        //{
        //    this.srvProvincia = new ProvinciaService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvProvincia">Interfaz de servicio de provincia</param>
        /// <returns></returns>
        public ProvinciaController(IProvinciaService srvProvincia)
        {
            this.srvProvincia = srvProvincia;
        }

        #region IProvinciaController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de provincia
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de provincia</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdProvincia", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var provincias = new ProvinciaViewModel
                                 {
                                     Provincias =
                                         srvProvincia.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                     FilasTotal = srvProvincia.Contar(),
                                     FilasPorPagina = FilasPorPagina,
                                     TextoBusqueda = textoBuscar
                                 };

            // Devuelve una vista con los datos
            return View(provincias);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva provincia
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la provincia por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir una nueva provincia
            return View(new Provincia());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de una nueva provincia
        /// </summary>
        /// <param name="entity">Provincia actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(Provincia entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvProvincia.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una provincia
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la provincia</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Provincia provincia = srvProvincia.ObtenerId(id);

            // Si no la encuentra muestra una vista de error
            if (provincia == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(provincia);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la provincia
        /// </summary>
        /// <param name="entity">Provincia actual</param>
        [HttpPost]
        public ActionResult Editar(Provincia entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvProvincia.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una provincia a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la provincia, sino muestra una vista con el error
            if (!srvProvincia.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}