﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de rol de usuario. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IRolController : IBaseController<Rol, int>
    {
    }

    /// <summary>
    ///  Controlador de rol de usuario. Hereda del controlador base e implementa la interfaz de controlador de rol.
    /// </summary>
    [Authorize(Roles = "admin")]
    public class RolController : BaseController, IRolController
    {
        private const int FilasPorPagina = 10; // Número de filas por página 
        private readonly IRolService srvRol; // Interfaz del servicio de datos de rol

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public RolController()
        //{
        //    this.service = new RolService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvRol">Interfaz de servicio de rol de usuario</param>
        /// <returns></returns>
        public RolController(IRolService srvRol)
        {
            this.srvRol = srvRol;
        }

        #region IRolController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de rol de usuario
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de rol de usuario</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdRol", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var roles = new RolViewModel
                            {
                                Roles = srvRol.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                FilasTotal = srvRol.Contar(),
                                FilasPorPagina = FilasPorPagina,
                                TextoBusqueda = textoBuscar
                            };

            // Devuelve una vista con los datos
            return View(roles);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo rol de usuario
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del rol de usuario por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir un nuevo rol de usuario
            return View(new Rol());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de una nuevo rol de usuario
        /// </summary>
        /// <param name="entity">Rol de usuario actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(Rol entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvRol.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un rol de usuario
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del rol de usuario</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Rol rol = srvRol.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (rol == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(rol);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del rol de usuario
        /// </summary>
        /// <param name="entity">Rol de usuario actual</param>
        [HttpPost]
        public ActionResult Editar(Rol entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvRol.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un rol de usuario a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el rol de usuario, sino muestra una vista con el error
            if (!srvRol.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}