﻿using System;
using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de seguimiento de entidad. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface ISeguimientoController : IBaseController<Seguimiento, int>
    {
    }

    /// <summary>
    /// Controlador de seguimiento. Hereda del controlador base e implementa la interfaz de controlador de seguimiento.
    /// </summary>
    [Authorize]
    public class SeguimientoController : BaseController, ISeguimientoController
    {
        private const int FilasPorPagina = 10;                  // Número de filas por página
        private readonly ISeguimientoService srvSeguimiento;    // Interfaz del servicio de datos de seguimientos
        private readonly IEntidadService srvEntidad;            // Interfaz del servicio de datos de entidades       
        private readonly IUsuarioService srvUsuario;            // Interfaz del servicio de datos de usuarios       
        private readonly IMedioService srvMedio;                // Interfaz del servicio de datos de medios       
        private readonly IEstadoService srvEstado;              // Interfaz del servicio de datos de estados       

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public SeguimientoController()
        //{
        //    this.srvSeguimiento = new SeguimientoService();
        //    this.srvEntidad = new EntidadService().ObtenerTodos();
        //    this.srvUsuario = new UsuarioService().ObtenerTodos();
        //    this.srvMedio = new MEdioService().ObtenerTodos();
        //    this.srvEstado = new EstadoService().ObtenerTodos();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvSeguimiento">Interfaz de servicio de seguimiento</param>
        /// <param name="srvEntidad">Interfaz de servicio de entidad</param>
        /// <param name="srvUsuario">Interfaz de servicio de usuario</param>
        /// <param name="srvMedio">Interfaz de servicio de medio</param>
        /// <param name="srvEstado">Interfaz de servicio de estado</param>
        /// <returns></returns>
        public SeguimientoController(ISeguimientoService srvSeguimiento, IEntidadService srvEntidad, 
                                     IUsuarioService srvUsuario, IMedioService srvMedio, IEstadoService srvEstado)
        {
            this.srvSeguimiento = srvSeguimiento;
            this.srvEntidad = srvEntidad;
            this.srvUsuario = srvUsuario;
            this.srvMedio = srvMedio;
            this.srvEstado = srvEstado;
        }

        #region ISeguimientoController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del seguimiento
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del seguimiento</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdSeguimiento", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var seguimientos = new SeguimientoViewModel
                               {   Seguimientos = srvSeguimiento.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvSeguimiento.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(seguimientos);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo seguimiento
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del seguimiento por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea un modelo de la vista para editar el nuevo seguimiento
            var seguimientos = new SeguimientoViewModelEdit
                               {
                                   Seguimiento = new Seguimiento(),
                                   Entidades = srvEntidad.ObtenerTodos(),
                                   Usuarios = srvUsuario.ObtenerTodos(),
                                   Medios = srvMedio.ObtenerTodos(),
                                   Estados = srvEstado.ObtenerTodos()
                               };

            // Se definen las propiedades por defecto
            seguimientos.Seguimiento.Fecha = DateTime.Now;  // Por defecto un nuevo seguimiento es de la fecha actual
            seguimientos.Seguimiento.IdEntidad = 1;         // Por defecto un nuevo seguimiento es de la primera entidad
            seguimientos.Seguimiento.IdUsuario = 1;         // Por defecto un nuevo seguimiento es de el primer usuario
            seguimientos.Seguimiento.IdMedio = 1;           // Por defecto un nuevo seguimiento es de el primer medio
            seguimientos.Seguimiento.IdEstado = 1;          // Por defecto un nuevo seguimiento es de el primer estado

            // Devuelve una vista para añadir un nuevo seguimiento
            return View(seguimientos);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo seguimiento
        /// </summary>
        /// <param name="entity">Seguimiento actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Seguimiento")] Seguimiento entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var seguimientos = new SeguimientoViewModelEdit
                                   {
                                       Seguimiento = entity,
                                       Entidades = srvEntidad.ObtenerTodos(),
                                       Usuarios = srvUsuario.ObtenerTodos(),
                                       Medios = srvMedio.ObtenerTodos(),
                                       Estados = srvEstado.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(seguimientos);
            }

            // Actualiza los datos
            if (!srvSeguimiento.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un seguimiento
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del seguimiento</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Seguimiento seguimiento = srvSeguimiento.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (seguimiento == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var seguimientos = new SeguimientoViewModelEdit
                               {
                                   Seguimiento = seguimiento,
                                   Entidades = srvEntidad.ObtenerTodos(),
                                   Usuarios = srvUsuario.ObtenerTodos(),
                                   Medios = srvMedio.ObtenerTodos(),
                                   Estados = srvEstado.ObtenerTodos()
                               };

            // Devuelve una vista para editar
            return View(seguimientos);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del seguimiento
        /// </summary>
        /// <param name="entity">Seguimiento actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Seguimiento")] Seguimiento entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var seguimientos = new SeguimientoViewModelEdit
                                   {
                                       Seguimiento = entity,
                                       Entidades = srvEntidad.ObtenerTodos(),
                                       Usuarios = srvUsuario.ObtenerTodos(),
                                       Medios = srvMedio.ObtenerTodos(),
                                       Estados = srvEstado.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(seguimientos);
            }

            // Actualiza los datos
            if (!srvSeguimiento.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un seguimiento a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el seguimiento, sino muestra una vista con el error
            if (id <= 1 || !srvSeguimiento.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

    }
}