﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    /// Interfaz de controlador de temática de acción. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface ITematicaController : IBaseController<Tematica, int>
    {
    }

    /// <summary>
    /// Controlador de temática de acción. Hereda del controlador base e implementa la interfaz de controlador de temática.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class TematicaController : BaseController, ITematicaController
    {
        public const int FilasPorPagina = 10; // Número de filas por página 
        private readonly ITematicaService srvTematica; // Interfaz del servicio de datos de temática de acción

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public TematicaController()
        //{
        //    this.srvTematica = new TematicaService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvTematica">Interfaz de servicio de temática de acción</param>
        /// <returns></returns>
        public TematicaController(ITematicaService srvTematica)
        {
            this.srvTematica = srvTematica;
        }

        #region ITematicaController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de la temática de acción
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de la temática de acción</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdTematica", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var tematicas = new TematicaViewModel
                                {
                                    Tematicas =
                                        srvTematica.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                    FilasTotal = srvTematica.Contar(),
                                    FilasPorPagina = FilasPorPagina,
                                    TextoBusqueda = textoBuscar
                                };

            // Devuelve una vista con los datos
            return View(tematicas);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir una nueva temática de acción
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos de la temática de acción por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir una nueva temática de acción
            return View(new Tematica());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de una nueva temática de acción
        /// </summary>
        /// <param name="entity">Temática de acción actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(Tematica entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTematica.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar una temática de acción
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos de la temática de acción</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Tematica tematica = srvTematica.ObtenerId(id);

            // Si no la encuentra muestra una vista de error
            if (tematica == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(tematica);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de la temática de acción
        /// </summary>
        /// <param name="entity">Temática de acción actual</param>
        [HttpPost]
        public ActionResult Editar(Tematica entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTematica.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina una temática de acción a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina la  temática de acción, sino muestra una vista con el error
            if (!srvTematica.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}