﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de tipo de entidad. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface ITipoEntidadController : IBaseController<TipoEntidad, int>
    {
    }

    /// <summary>
    ///  Controlador de tipo de entidad. Hereda del controlador base e implementa la interfaz de controlador de tipo de entidad.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class TipoEntidadController : BaseController, ITipoEntidadController
    {
        public const int FilasPorPagina = 10; // Número de filas por página 
        private readonly ITipoEntidadService srvTipoEntidad; // Interfaz del servicio de datos de tipo de entidad

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public TipoEntidadController()
        //{
        //    this.srvTipoEntidad = new TipoEntidadService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvTipoEntidad">Interfaz de servicio de tipo de entidad</param>
        /// <returns></returns>
        public TipoEntidadController(ITipoEntidadService srvTipoEntidad)
        {
            this.srvTipoEntidad = srvTipoEntidad;
        }

        #region ITipoEntidadController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de tipo de entidad
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de tipo de entidad</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdTipoEntidad", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var tiposentidades = new TipoEntidadViewModel
                                     {
                                         TiposEntidades =
                                             srvTipoEntidad.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir,
                                                                          textoBuscar),
                                         FilasTotal = srvTipoEntidad.Contar(),
                                         FilasPorPagina = FilasPorPagina,
                                         TextoBusqueda = textoBuscar
                                     };

            // Devuelve una vista con los datos
            return View(tiposentidades);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo tipo de entidad
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del  tipo de entidad por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir un nuevo tipo de entidad
            return View(new TipoEntidad());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de un nuevo tipo de entidad
        /// </summary>
        /// <param name="entity">Tipo de entidad actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(TipoEntidad entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTipoEntidad.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un tipo de entidad
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del tipo de entidad</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            TipoEntidad tipoentidad = srvTipoEntidad.ObtenerId(id);

            // Si no la encuentra muestra una vista de error
            if (tipoentidad == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(tipoentidad);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de un tipo de entidad
        /// </summary>
        /// <param name="entity">Tipo de entidad actual</param>
        [HttpPost]
        public ActionResult Editar(TipoEntidad entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTipoEntidad.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un tipo de entidad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el tipo de entidad, sino muestra una vista con el error
            if (!srvTipoEntidad.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}