﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de tipos de organización. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface ITipoOrganizacionController : IBaseController<TipoOrganizacion, int>
    {
    }

    /// <summary>
    ///  Controlador de tipos de organización. Hereda del controlador base e implementa la interfaz de controlador de tipos de organización.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class TipoOrganizacionController : BaseController, ITipoOrganizacionController
    {
        public const int FilasPorPagina = 10; // Número de filas por página 
        private readonly ITipoOrganizacionService srvTipoOrganizacion; // Interfaz del servicio de datos de provincia

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public TipoOrganizacionController()
        //{
        //    this.srvTipoOrganizacion = new TipoOrganizacionService();
        //}
        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvTipoOrganizacion">Interfaz de servicio de tipos de organización</param>
        /// <returns></returns>
        public TipoOrganizacionController(ITipoOrganizacionService srvTipoOrganizacion)
        {
            this.srvTipoOrganizacion = srvTipoOrganizacion;
        }

        #region ITipoOrganizacionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del tipo de organización
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del tipo de organización</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdTipoOrganizacion", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var tiposorgs = new TipoOrganizacionViewModel
                                {
                                    TiposOrganizaciones =
                                        srvTipoOrganizacion.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir,
                                                                          textoBuscar),
                                    FilasTotal = srvTipoOrganizacion.Contar(),
                                    FilasPorPagina = FilasPorPagina,
                                    TextoBusqueda = textoBuscar
                                };

            // Devuelve una vista con los datos
            return View(tiposorgs);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo tipo de organización
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del tipo de organización por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Devuelve una vista para añadir un nuevo tipo de organización
            return View(new TipoOrganizacion());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo tipo de organización
        /// </summary>
        /// <param name="entity">Tipo de organización actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo(TipoOrganizacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTipoOrganizacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un tipo de organización
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del usuario</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            TipoOrganizacion tipoorg = srvTipoOrganizacion.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (tipoorg == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(tipoorg);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del tipo de organización
        /// </summary>
        /// <param name="entity">Tipo de organización actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>  
        [HttpPost]
        public ActionResult Editar(TipoOrganizacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTipoOrganizacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un tipo de organización a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el usuario, sino muestra una vista con el error
            if (!srvTipoOrganizacion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}