﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    /// Interfaz de controlador de tipo de participación. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface ITipoParticipacionController : IBaseController<TipoParticipacion, int>
    {
    }

    /// <summary>
    ///  Controlador de tipo de participación. Hereda del controlador base e implementa la interfaz de controlador de tipo de participación.
    /// </summary>
    [Authorize(Roles = "admin, manager")]
    public class TipoParticipacionController : BaseController, ITipoParticipacionController
    {
        // Número de filas por página 
        private const int FilasPorPagina = 10; 

        // Interfaz del servicio de datos de tipo de participación
        private readonly ITipoParticipacionService srvTipoParticipacion;                                                 

        /// <summary>
        ///  Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public TipoParticipacionController()
        //{
        //    this.srvTipoParticipacion = new TipoParticipacionService();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvTipoParticipacion">Interfaz de servicio de tipo de participación</param>
        /// <returns></returns>
        public TipoParticipacionController(ITipoParticipacionService srvTipoParticipacion)
        {
            this.srvTipoParticipacion = srvTipoParticipacion;
        }

        #region ITipoParticipacionController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos de tipo de participación
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos de tipo de participación</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdTipoParticipacion", string ordenDir = "asc", string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var tiposparticipaciones = new TipoParticipacionViewModel
                                           {
                                               TiposParticipaciones = srvTipoParticipacion.ObtenerFiltro(pagina, FilasPorPagina, orden,
                                                                                      ordenDir, textoBuscar),
                                               FilasTotal = srvTipoParticipacion.Contar(),
                                               FilasPorPagina = FilasPorPagina,
                                               TextoBusqueda = textoBuscar
                                           };

            // Devuelve una vista con los datos
            return View(tiposparticipaciones);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo tipo de participación
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del  tipo de participación por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            return View(new TipoParticipacion());
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos de un nuevo tipo de participación
        /// </summary>
        /// <param name="entity">Tipo de participación actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns> 
        [HttpPost]
        public ActionResult Nuevo(TipoParticipacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTipoParticipacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un tipo de participación
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del tipo de participación</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            TipoParticipacion tipoparticipacion = srvTipoParticipacion.ObtenerId(id);

            // Si no la encuentra muestra una vista de error
            if (tipoparticipacion == null)
                return View("Error");

            // Devuelve una vista para editar
            return View(tipoparticipacion);
        }


        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos de un tipo de participación
        /// </summary>
        /// <param name="entity">Tipo de participación actual</param>
        [HttpPost]
        public ActionResult Editar(TipoParticipacion entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Vuelve a editar el modelo y muestra los errores
                return View(entity);
            }

            // Actualiza los datos
            if (!srvTipoParticipacion.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un tipo de participación a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el tipo de participación, sino muestra una vista con el error
            if (!srvTipoParticipacion.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion
    }
}