﻿using System.Web.Mvc;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.ViewModels;

namespace GPAS.Controllers
{
    /// <summary>
    ///  Interfaz de controlador de usuario. Hereda de la interfaz del controlador base y añade los metodos necesarios.
    /// </summary>
    public interface IUsuarioController : IBaseController<Usuario, int>
    {
    }

    /// <summary>
    /// Controlador de usuario. Hereda del controlador base e implementa la interfaz de controlador de usuario.
    /// </summary>
    [Authorize(Roles = "admin")]
    public class UsuarioController : BaseController, IUsuarioController
    {
        private const int FilasPorPagina = 10; // Número de filas por página
        private readonly IDelegacionService srvDelegacion; // Interfaz del servicio de datos de delegaciones
        private readonly IRolService srvRol; // Interfaz del servicio de datos de roles
        private readonly IUsuarioService srvUsuario; // Interfaz del servicio de datos de usuario

        /// <summary>
        /// Constructor por defecto (eliminado para permitir Inyección de dependencias)
        /// </summary>
        //public UsuarioController()
        //{
        //    this.srvUsuario = new UsuarioService();
        //    this.srvRol = new RolService().GetAll();
        //    this.srvDelegacion = new DelegacionService().GetAll();
        //}

        /// <summary>
        /// Constructor que permite la inyección de dependencias
        /// </summary>
        /// <param name="srvUsuario">Interfaz de servicio de usuario</param>
        /// <param name="srvRol">Interfaz de servicio de rol</param>
        /// <param name="srvDelegacion">Interfaz de servicio de delegacion</param>
        /// <returns></returns>
        public UsuarioController(IUsuarioService srvUsuario, IRolService srvRol, IDelegacionService srvDelegacion)
        {
            this.srvUsuario = srvUsuario;
            this.srvRol = srvRol;
            this.srvDelegacion = srvDelegacion;
        }

        #region IUsuarioController Members

        /// <summary>
        /// Accion Indice. Devuelve una vista con los datos del usuario
        /// </summary>
        /// <param name="pagina">Numero de pagina</param>
        /// <param name="orden">Nombre del campo de ordenación</param>
        /// <param name="ordenDir">Direccion de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve la vista 'Indice' con los datos del usuario</returns>
        public ActionResult Indice(int pagina = 1, string orden = "IdUsuario", string ordenDir = "asc",
                                   string textoBuscar = "")
        {
            // Usamos el modelo para obtener los datos e introducirlos en el objeto ViewModel que enviamos a la vista
            var usuarios = new UsuarioViewModel
                               {
                                   Usuarios =
                                       srvUsuario.ObtenerFiltro(pagina, FilasPorPagina, orden, ordenDir, textoBuscar),
                                   FilasTotal = srvUsuario.Contar(),
                                   FilasPorPagina = FilasPorPagina,
                                   TextoBusqueda = textoBuscar
                               };

            // Devuelve una vista con los datos
            return View(usuarios);
        }

        /// <summary>
        /// Accion Nuevo (Get). Devuelve una vista para añadir un nuevo usuario
        /// </summary>
        /// <returns>Devuelve la vista 'Nuevo' con los datos del usuario por defecto</returns>
        [HttpGet]
        public ActionResult Nuevo()
        {
            // Crea un modelo de la vista para editar el nuevo usuario
            var usuarios = new UsuarioViewModelEdit
                               {
                                   Usuario = new Usuario(),
                                   Delegaciones = srvDelegacion.ObtenerTodos(),
                                   Roles = srvRol.ObtenerTodos()
                               };

            // Se definen las propiedades por defecto
            usuarios.Usuario.IdDelegacion = 1; // Por defecto un nuevo usuario es de Cataluña
            usuarios.Usuario.IdRol = 4; // Por defecto un nuevo usuario tiene un rol de invitado
            usuarios.Usuario.NombreUsuario = ""; // Por defecto un nuevo usuario no tiene nombre
            usuarios.Usuario.Contrasena = ""; // Por defecto un nuevo usuario no tiene contraseña
            usuarios.Usuario.Activo = true; // Por defecto un nuevo usuario estará activo 

            // Devuelve una vista para añadir un nuevo usuario
            return View(usuarios);
        }

        /// <summary>
        /// Accion Nuevo (Post). Valida y actualiza los datos del nuevo usuario
        /// </summary>
        /// <param name="entity">Usuario actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Nuevo' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Nuevo([Bind(Prefix = "Usuario")] Usuario entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var usuarios = new UsuarioViewModelEdit
                                   {
                                       Usuario = entity,
                                       Delegaciones = srvDelegacion.ObtenerTodos(),
                                       Roles = srvRol.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(usuarios);
            }

            // Actualiza los datos
            if (!srvUsuario.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Editar (Get). Devuelve una vista para editar un usuario
        /// </summary>
        /// <param name="id">Identificador a editar</param>
        /// <returns>Devuelve la vista 'Editar' con los datos del usuario</returns>
        [HttpGet]
        public ActionResult Editar(int id)
        {
            // Obtiene los datos a partir del id
            Usuario usuario = srvUsuario.ObtenerId(id);

            // Si no lo encuentra muestra una vista de error
            if (usuario == null)
                return View("Error");

            // Crea el modelo de la vista para editar
            var usuarios = new UsuarioViewModelEdit
                               {
                                   Usuario = usuario,
                                   Delegaciones = srvDelegacion.ObtenerTodos(),
                                   Roles = srvRol.ObtenerTodos()
                               };

            // Devuelve una vista para editar
            return View(usuarios);
        }

        /// <summary>
        /// Accion Editar (Post). Valida y actualiza los datos del usuario
        /// </summary>
        /// <param name="entity">Usuario actual</param>
        /// <returns>Si es válido, actualiza y vuelve a la vista 'Indice', sino vuelve la vista 'Editar' mostrando los errores</returns>
        [HttpPost]
        public ActionResult Editar([Bind(Prefix = "Usuario")] Usuario entity)
        {
            // Si el modelo no es válido
            if (!ModelState.IsValid)
            {
                // Crea de nuevo el modelo de la vista con los datos anteriores para volver a editar
                var usuarios = new UsuarioViewModelEdit
                                   {
                                       Usuario = entity,
                                       Delegaciones = srvDelegacion.ObtenerTodos(),
                                       Roles = srvRol.ObtenerTodos()
                                   };

                // Vuelve a editar el modelo y muestra los errores
                return View(usuarios);
            }

            // Actualiza los datos
            if (!srvUsuario.Guardar(entity))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        /// <summary>
        /// Accion Borrar (Get). Elimina un usuario a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a eliminar</param>
        /// <returns>Si es válido lo elimina y vuelve a la vista 'Indice', sino muestra una vista con el error</returns>
        [Authorize(Roles = "admin, manager, user")]
        [HttpGet]
        public ActionResult Borrar(int id)
        {
            // Elimina el usuario, sino muestra una vista con el error
            if (id <= 1 || !srvUsuario.Borrar(id))
                return View("Error");

            // Vuelve al indice
            return RedirectToAction("Indice");
        }

        #endregion

        /// <summary>
        /// Accion ComprobarNombre (Post). Comprueba la existencia previa del nombre de usuario
        /// </summary>
        /// <param name="entity">Usuario actual</param>
        /// <returns>Devuelve 'true' si el usuario no existe previamente</returns>
        [HttpPost]
        public ActionResult ComprobarNombre([Bind(Prefix = "Usuario")] Usuario entity)
        {
            // Si no existe el nombre de usuario es válido
            return Json(!srvUsuario.ExisteNombre(entity.NombreUsuario), JsonRequestBehavior.AllowGet);

            /*
            // Se busca un nombre alternativo
            string sugerencia = String.Format(CultureInfo.InvariantCulture, "{0} no esta disponible.", nombre);

            for (int i = 1; i < 100; i++)
            {
                string alternativo = nombre + i.ToString();
                if (!srvUsuario.ExisteNombre(alternativo))
                {
                    sugerencia = String.Format(CultureInfo.InvariantCulture, "{0} no esta disponible. Pruebe {1}.", nombre, alternativo);
                    break;
                }
            }
            return Json(sugerencia, JsonRequestBehavior.AllowGet);
            */
        }
    }
}