﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web.Mvc;
using System.Web.Routing;
using System.Text;

namespace GPAS.Helpers
{
    public static class HtmlHelperExtensions
    {
        // LabelFor
        public static MvcHtmlString LabelFor<TModel, TValue>(this HtmlHelper<TModel> html,
                                                             Expression<Func<TModel, TValue>> expression,
                                                             object htmlAttributes)
        {
            return LabelFor(html, expression, new RouteValueDictionary(htmlAttributes));
        }

        public static MvcHtmlString LabelFor<TModel, TValue>(this HtmlHelper<TModel> html,
                                                             Expression<Func<TModel, TValue>> expression,
                                                             IDictionary<string, object> htmlAttributes)
        {
            ModelMetadata metadata = ModelMetadata.FromLambdaExpression(expression, html.ViewData);
            string htmlFieldName = ExpressionHelper.GetExpressionText(expression);
            string labelText = metadata.DisplayName ?? metadata.PropertyName ?? htmlFieldName.Split('.').Last();
            if (String.IsNullOrEmpty(labelText))
            {
                return MvcHtmlString.Empty;
            }

            var tag = new TagBuilder("label");
            tag.MergeAttributes(htmlAttributes);
            tag.Attributes.Add("for", html.ViewContext.ViewData.TemplateInfo.GetFullHtmlFieldId(htmlFieldName));
            tag.SetInnerText(labelText);
            return MvcHtmlString.Create(tag.ToString(TagRenderMode.Normal));
        }


        // SetFocus
        public static MvcHtmlString SetFocusTo<TModel, TProperty>(this HtmlHelper<TModel> html,
                                                                  Expression<Func<TModel, TProperty>> expression)
        {
            string prop = ExpressionHelper.GetExpressionText(expression);
            return html.setFocusTo(prop);
        }

        public static MvcHtmlString SetFocusTo(this HtmlHelper html, string propertyName)
        {
            string prop = ExpressionHelper.GetExpressionText(propertyName);
            return html.setFocusTo(prop);
        }

        private static MvcHtmlString setFocusTo(this HtmlHelper html, string property)
        {
            string id = html.ViewData.TemplateInfo.GetFullHtmlFieldId(property);
            string script = "<script type='text/javascript'>" +
                            "$(function() {" +
                            "$('#" + id + "').focus();" +
                            "});" +
                            "</script>";
            return MvcHtmlString.Create(script);
        }


        public static MvcHtmlString Image(this HtmlHelper helper, string url, string altText, object htmlAttributes)
        {
            var builder = new TagBuilder("img");
            builder.Attributes.Add("src", url);
            builder.Attributes.Add("alt", altText);
            builder.MergeAttributes(new RouteValueDictionary(htmlAttributes));
            return MvcHtmlString.Create(builder.ToString(TagRenderMode.SelfClosing));
        }

        public static MvcHtmlString Button(this HtmlHelper helper, string text,
                                           IDictionary<string, object> htmlAttributes)
        {
            var builder = new TagBuilder("button");
            builder.InnerHtml = text;
            builder.MergeAttributes(htmlAttributes);
            return MvcHtmlString.Create(builder.ToString());
        }

        public static MvcHtmlString ImageButton(this HtmlHelper helper, string name, object htmlAttributes,
                                                string imagePath)
        {
            var url = new UrlHelper(helper.ViewContext.RequestContext);

            // build the <img> tag
            var imgBuilder = new TagBuilder("img");
            imgBuilder.MergeAttribute("src", url.Content(imagePath));
            if (!string.IsNullOrWhiteSpace(name)) imgBuilder.MergeAttribute("alt", name);
            string imgHtml = imgBuilder.ToString(TagRenderMode.SelfClosing);

            var builder = new TagBuilder("button");
            builder.MergeAttributes(new RouteValueDictionary(htmlAttributes));
            builder.InnerHtml = imgHtml;
            builder.InnerHtml += "&nbsp;" + name;

            return MvcHtmlString.Create(builder.ToString());
        }


        public static MvcHtmlString ImageActionLink(this HtmlHelper helper, string name, string action,
                                                    object htmlAttributes, string imagePath)
        {
            return ImageActionLink(helper, name, action, null, null, htmlAttributes, imagePath);
        }

        public static MvcHtmlString ImageActionLink(this HtmlHelper helper, string name, string action,
                                                    string controller, object htmlAttributes, string imagePath)
        {
            return ImageActionLink(helper, name, action, controller, null, htmlAttributes, imagePath);
        }

        public static MvcHtmlString ImageActionLink(this HtmlHelper helper, string name, string action,
                                                    object routeValues, object htmlAttributes, string imagePath)
        {
            return ImageActionLink(helper, name, action, null, routeValues, htmlAttributes, imagePath);
        }

        public static MvcHtmlString ImageActionLink(this HtmlHelper helper, string name, string action,
                                                    string controller, object routeValues, object htmlAttributes, string imagePath)
        {
            var url = new UrlHelper(helper.ViewContext.RequestContext);

            // Construir la etiqueta <img> 
            var imgBuilder = new TagBuilder("img");
            imgBuilder.MergeAttribute("src", url.Content(imagePath));
            if (!string.IsNullOrWhiteSpace(name)) imgBuilder.MergeAttribute("alt", name);
            string imgHtml = imgBuilder.ToString(TagRenderMode.SelfClosing);

            // Construir la etiqueta <a>           
            var anchorBuilder = new TagBuilder("a");
            anchorBuilder.MergeAttribute("href", url.Action(action, controller, routeValues));
            anchorBuilder.MergeAttributes(new RouteValueDictionary(htmlAttributes));

            // Incluir la etiqueta <img> en el interior
            anchorBuilder.InnerHtml = imgHtml;
            if (!string.IsNullOrWhiteSpace(name)) anchorBuilder.InnerHtml += "&nbsp;" + name;

            return MvcHtmlString.Create(anchorBuilder.ToString(TagRenderMode.Normal));
        }

        public static MvcHtmlString AutoCompleter(this HtmlHelper helper, string name, string controller, string action,
                                                  string fieldName, bool mustMatch)
        {
            StringBuilder html = new StringBuilder("<script type=\"text/javascript\"> \n ");
            html.Append("\t $(document).ready( function() { \n");
            html.AppendLine("\t\t $('#" + name + "').autocomplete('/" + controller + "/" + action + "/', {");
            html.AppendLine("\t\t dataType: 'json',");
            html.AppendLine("\t\t parse: function(data) {");
            html.AppendLine("\t\t var rows = new Array();");
            html.AppendLine("\t\t for(var i=0; i<data.length; i++){");
            html.AppendLine("\t\t rows[i] = { data:data[i], value:data[i]." + fieldName + ", result:data[i]." + fieldName + " };");
            html.AppendLine("\t\t } return rows;},");
            html.AppendLine("\t\t formatItem: function(row, i, n) {");
            html.AppendLine("\t\t return row." + fieldName + ";},");
            html.AppendLine("\t\t width: 300, mustMatch: " + mustMatch.ToString().ToLower() + " , });}); \n ");
            html.AppendLine("\t $(function () { \n\t $('#" + name + "').autocomplete().result(function(event, data, formatted) { ");
            html.AppendLine("\t\t $('#" + controller + "_Id" + name + "').val(data.Id); }); }); \n </script>");
            return MvcHtmlString.Create(html.ToString());
        }

    }
}