﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de actividad. Implementa los métodos de la interfaz del servicio de actividad
    /// </summary>
    public class ActividadService : IActividadService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IActividadService Members

        /// <summary>
        /// Guarda los datos de actividad
        /// </summary>
        /// <param name="entity">Actividad a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Actividad entity)
        {
            try
            {
                if (entity.IdActividad == 0)
                {
                    // Si es nuevo
                    datos.AddToActividades(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Actividades.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de actividad
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Actividad actividad = datos.Actividades.FirstOrDefault(m => m.IdActividad == id);
            if (actividad == null)
                return false;

            datos.Actividades.DeleteObject(actividad);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el numero de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos </returns>
        public int Contar()
        {
            return datos.Actividades.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de actividades 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de las actividades</returns>
        public IEnumerable<Actividad> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                    string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idactividad", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var actividades =
                (IQueryable<Actividad>) datos.Actividades.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                actividades = actividades.Where(a => a.Descripcion.Contains(textoBuscar));

            return actividades.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de actividades 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de las actividades</returns>
        public IEnumerable<Actividad> ObtenerTodos(int pagina, int filas, Func<Actividad, IComparable> funcOrden,
                                                   Orden orden)
        {
            IEnumerable<Actividad> actividades;
            if (orden == Orden.Ascendente)
                actividades = datos.Actividades.OrderBy(funcOrden);
            else
                actividades = datos.Actividades.OrderByDescending(funcOrden);

            return actividades.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de actividades 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de las actividades</returns>
        public IEnumerable<Actividad> ObtenerTodos()
        {
            return datos.Actividades.ToList();
        }

        /// <summary>
        /// Obtiene una actividad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una actividad</returns>
        public Actividad ObtenerId(int id)
        {
            return datos.Actividades.FirstOrDefault(m => m.IdActividad == id);
        }

        #endregion
    }
}