﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de contacto de entidad. Implementa los métodos de la interfaz del servicio de contacto
    /// </summary>
    public class ContactoService : IContactoService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region Miembros de IBaseService<Contacto,int>

        /// <summary>
        /// Guarda los datos de contacto
        /// </summary>
        /// <param name="entity">Contacto a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Contacto entity)
        {
            try
            {
                if (entity.IdContacto == 0)
                {
                    // Si es nuevo
                    datos.AddToContactos(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Contactos.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de contacto
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Contacto contacto = datos.Contactos.FirstOrDefault(m => m.IdContacto == id);
            if (contacto == null)
                return false;

            datos.Contactos.DeleteObject(contacto);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Contactos.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de contactos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de las contactos</returns>
        public IEnumerable<Contacto> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                   string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase) ? sentidoOrdenacion : "asc";
            var columnasValidas = new[] { "idcontacto", "entidad.nombre", "nombre", "apellidos", "cargo", "email",
                                          "telefono", "extension" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var contactos =
                (IQueryable<Contacto>) datos.Contactos.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                contactos = contactos.Where(c => c.Nombre.Contains(textoBuscar) || c.Apellidos.Contains(textoBuscar) ||
                                                 c.Cargo.Contains(textoBuscar) || c.Email.Contains(textoBuscar) ||
                                                 c.Telefono.Contains(textoBuscar) || c.Extension.Contains(textoBuscar) ||
                                                 c.Entidad.Nombre.Contains(textoBuscar));

            return contactos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de contactos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de las contactos</returns>
        public IEnumerable<Contacto> ObtenerTodos(int pagina, int filas, Func<Contacto, IComparable> funcOrden,
                                                  Orden orden)
        {
            IEnumerable<Contacto> contactos;
            if (orden == Orden.Ascendente)
                contactos = datos.Contactos.OrderBy(funcOrden);
            else
                contactos = datos.Contactos.OrderByDescending(funcOrden);

            return contactos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de contactos 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de contactos</returns>
        public IEnumerable<Contacto> ObtenerTodos()
        {
            return datos.Contactos.ToList();
        }

        /// <summary>
        /// Obtiene un contacto a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un contacto</returns>
        public Contacto ObtenerId(int id)
        {
            return datos.Contactos.FirstOrDefault(c => c.IdContacto == id);
        }

        #endregion

        #region Miembros de IContactoService

        /// <summary>
        /// Obtiene una lista con los contactos por entidad
        /// </summary>
        /// <param name="identidad">Identificador de la entidad</param>
        /// <returns>Devuelve los contactos por entidad</returns>
        public IEnumerable<Contacto> ObtenerEntidad(int identidad)
        {
            return datos.Contactos.Where(c => c.IdEntidad.Equals(identidad)).ToList();
        }

        #endregion
    }
}