﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de delegación. Implementa los métodos de la interfaz del servicio de delegación
    /// </summary>
    public class DelegacionService : IDelegacionService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region Miembros de IBaseService<Delegacion,int>

        /// <summary>
        /// Guarda los datos de delegación
        /// </summary>
        /// <param name="entity">Delegación a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Delegacion entity)
        {
            try
            {
                if (entity.IdDelegacion == 0)
                {
                    // Si es nuevo
                    datos.AddToDelegaciones(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Delegaciones.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de delegación
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Delegacion delegacion = datos.Delegaciones.FirstOrDefault(m => m.IdDelegacion == id);
            if (delegacion == null)
                return false;

            datos.Delegaciones.DeleteObject(delegacion);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Delegaciones.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de delegaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de delegaciones</returns>
        public IEnumerable<Delegacion> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                     string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase) ? sentidoOrdenacion : "asc";
            var columnasValidas = new[] { "iddelegacion", "nombre", "responsable", "telefono", "email", "direccion", 
                                          "codigopostal", "organizacion.nombre", "poblacion.provincia.nombre", "poblacion.nombre" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var delegaciones =
                (IQueryable<Delegacion>) datos.Delegaciones.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                delegaciones = delegaciones.Where(d => d.Nombre.Contains(textoBuscar) || d.Responsable.Contains(textoBuscar) ||
                                                  d.Telefono.Contains(textoBuscar) || d.EMail.Contains(textoBuscar) ||
                                                  d.Direccion.Contains(textoBuscar) || d.CodigoPostal.Contains(textoBuscar) ||
                                                  d.Organizacion.Nombre.Contains(textoBuscar) || d.Poblacion.Provincia.Nombre.Contains(textoBuscar) ||
                                                  d.Poblacion.Nombre.Contains(textoBuscar));

            return delegaciones.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de delegaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de delegaciones</returns>
        public IEnumerable<Delegacion> ObtenerTodos(int pagina, int filas, Func<Delegacion, IComparable> funcOrden,
                                                    Orden orden)
        {
            IEnumerable<Delegacion> delegaciones;
            if (orden == Orden.Ascendente)
                delegaciones = datos.Delegaciones.OrderBy(funcOrden);
            else
                delegaciones = datos.Delegaciones.OrderByDescending(funcOrden);

            return delegaciones.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de delegaciones 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de delegaciones</returns>
        public IEnumerable<Delegacion> ObtenerTodos()
        {
            return datos.Delegaciones.ToList();
        }

        /// <summary>
        /// Obtiene una delegación a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una delegación</returns>
        public Delegacion ObtenerId(int id)
        {
            return datos.Delegaciones.FirstOrDefault(m => m.IdDelegacion == id);
        }

        #endregion

        #region Miembros de IDelegacionService

        #endregion
    }
}