﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de documento de evento. Implementa los métodos de la interfaz del servicio de documento
    /// </summary>
    public class DocumentoService : IDocumentoService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region Miembros de IBaseService<Documento,int>

        /// <summary>
        /// Guarda los datos de documento
        /// </summary>
        /// <param name="entity">Documento a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Documento entity)
        {
            try
            {
                if (entity.IdDocumento == 0)
                {
                    // Si es nuevo
                    datos.AddToDocumentos(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Documentos.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de Documento
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Documento documento = datos.Documentos.FirstOrDefault(m => m.IdDocumento == id);
            if (documento == null)
                return false;

            datos.Documentos.DeleteObject(documento);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Documentos.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de documentos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de las documentos</returns>
        public IEnumerable<Documento> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                    string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] { "iddocumento", "nombre", "fecha", "descripcion", "evento.nombre" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var documentos =
                (IQueryable<Documento>) datos.Documentos.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                documentos = documentos.Where(d => d.Nombre.Contains(textoBuscar) || d.Descripcion.Contains(textoBuscar) ||
                                              d.Evento.Nombre.Contains(textoBuscar)); // d.Fecha.ToString().Contains(textoBuscar));

            return documentos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de documentos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de las documentos</returns>
        public IEnumerable<Documento> ObtenerTodos(int pagina, int filas, Func<Documento, IComparable> funcOrden,
                                                   Orden orden)
        {
            IEnumerable<Documento> documentos;
            if (orden == Orden.Ascendente)
                documentos = datos.Documentos.OrderBy(funcOrden);
            else
                documentos = datos.Documentos.OrderByDescending(funcOrden);

            return documentos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de documentos 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de documentos</returns>
        public IEnumerable<Documento> ObtenerTodos()
        {
            return datos.Documentos.ToList();
        }

        /// <summary>
        /// Obtiene un documento a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un documento</returns>
        public Documento ObtenerId(int id)
        {
            return datos.Documentos.FirstOrDefault(m => m.IdDocumento == id);
        }

        #endregion

        #region Miembros de IDocumentoService

        /// <summary>
        /// Obtiene una lista con los documentos por evento
        /// </summary>
        /// <param name="idevento">Identificador del evento</param>
        /// <returns>Devuelve los documentos por evento</returns>
        public IEnumerable<Documento> ObtenerEvento(int idevento)
        {
            return datos.Documentos.Where(d => d.IdEvento.Equals(idevento)).ToList();
        }

        #endregion
    }
}