﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de entidad de colaboración. Implementa los métodos de la interfaz del servicio de entidad
    /// </summary>
    public class EntidadService : IEntidadService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IEntidadService Members

        /// <summary>
        /// Guarda los datos de entidad
        /// </summary>
        /// <param name="entity">Entidad a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Entidad entity)
        {
            try
            {
                if (entity.IdEntidad == 0)
                {
                    // Si es nuevo
                    datos.AddToEntidades(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Entidades.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de entidad
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Entidad entidad = datos.Entidades.FirstOrDefault(m => m.IdEntidad == id);
            if (entidad == null)
                return false;

            datos.Entidades.DeleteObject(entidad);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Entidades.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de entidades 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de entidades</returns>
        public IEnumerable<Entidad> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                  string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] { "identidad", "nombre", "email", "web", "direccion",
                                          "codigopostal", "telefono1", "telefono2", "movil", "observaciones",
                                          "tipoentidad.descripcion", "poblacion.provincia.nombre", "poblacion.nombre" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var entidades =
                (IQueryable<Entidad>) datos.Entidades.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                entidades = entidades.Where(d => d.Nombre.Contains(textoBuscar) || d.EMail.Contains(textoBuscar) ||
                                                 d.Web.Contains(textoBuscar) || d.Direccion.Contains(textoBuscar) ||
                                                 d.CodigoPostal.Contains(textoBuscar) || d.Observaciones.Contains(textoBuscar) ||
                                                 d.Telefono1.Contains(textoBuscar) || d.Telefono2.Contains(textoBuscar) ||
                                                 d.Movil.Contains(textoBuscar) || d.TipoEntidad.Descripcion.Contains(textoBuscar) ||
                                                 d.Poblacion.Provincia.Nombre.Contains(textoBuscar) || 
                                                 d.Poblacion.Nombre.Contains(textoBuscar));

            return entidades.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de entidades 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de entidades</returns>
        public IEnumerable<Entidad> ObtenerTodos(int pagina, int filas, Func<Entidad, IComparable> funcOrden,
                                                 Orden orden)
        {
            IEnumerable<Entidad> entidades;
            if (orden == Orden.Ascendente)
                entidades = datos.Entidades.OrderBy(funcOrden);
            else
                entidades = datos.Entidades.OrderByDescending(funcOrden);

            return entidades.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de entidades 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de entidades</returns>
        public IEnumerable<Entidad> ObtenerTodos()
        {
            return datos.Entidades.ToList();
        }

        /// <summary>
        /// Obtiene una entidad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una entidad</returns>
        public Entidad ObtenerId(int id)
        {
            return datos.Entidades.FirstOrDefault(m => m.IdEntidad == id);
        }

        #endregion
    }
}