﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de estado de entidad. Implementa los métodos de la interfaz del servicio de estado
    /// </summary>
    public class EstadoService : IEstadoService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IEstadoService Members

        /// <summary>
        /// Guarda los datos de estado
        /// </summary>
        /// <param name="entity">Estado a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Estado entity)
        {
            try
            {
                if (entity.IdEstado == 0)
                {
                    // Si es nuevo
                    datos.AddToEstados(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Estados.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de estado
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Estado estado = datos.Estados.FirstOrDefault(m => m.IdEstado == id);
            if (estado == null)
                return false;

            datos.Estados.DeleteObject(estado);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Estados.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de estados 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de estados</returns>
        public IEnumerable<Estado> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                 string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idestado", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var estados =
                (IQueryable<Estado>) datos.Estados.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                estados = estados.Where(e => e.Descripcion.Contains(textoBuscar));

            return estados.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de estados 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de estados</returns>
        public IEnumerable<Estado> ObtenerTodos(int pagina, int filas, Func<Estado, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Estado> estados;
            if (orden == Orden.Ascendente)
                estados = datos.Estados.OrderBy(funcOrden);
            else
                estados = datos.Estados.OrderByDescending(funcOrden);

            return estados.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de estados 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de estados</returns>
        public IEnumerable<Estado> ObtenerTodos()
        {
            return datos.Estados.ToList();
        }

        /// <summary>
        /// Obtiene una acción a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una acción</returns>
        public Estado ObtenerId(int id)
        {
            return datos.Estados.FirstOrDefault(m => m.IdEstado == id);
        }

        #endregion
    }
}