﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;
using GPAS.Utility; 

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de evento de acción. Implementa los métodos de la interfaz del servicio de evento
    /// </summary>
    public class EventoService : IEventoService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IEventoService Members

        /// <summary>
        /// Guarda los datos de evento
        /// </summary>
        /// <param name="entity">Evento a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Evento entity)
        {
            try
            {
                if (entity.IdEvento == 0)
                {
                    // Si es nuevo
                    datos.AddToEventos(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Eventos.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de evento
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Evento evento = datos.Eventos.FirstOrDefault(m => m.IdEvento == id);
            if (evento == null)
                return false;

            datos.Eventos.DeleteObject(evento);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Eventos.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de eventos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de eventos</returns>
        public IEnumerable<Evento> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                 string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase) ? sentidoOrdenacion : "asc";
            var columnasValidas = new[] { "idevento", "accion.organizacion.nombre", "accion.nombre", "poblacion.provincia.nombre", 
                                          "poblacion.nombre", "nombre", "direccion", "codigopostal", "observaciones",
                                          "fechainicio", "fechafin" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var eventos =
                (IQueryable<Evento>) datos.Eventos.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                eventos = eventos.Where(e => e.Nombre.Contains(textoBuscar) || e.Direccion.Contains(textoBuscar) ||
                                             e.CodigoPostal.Contains(textoBuscar) ||
                                             e.Observaciones.Contains(textoBuscar) ||
                                             e.Accion.Organizacion.Nombre.Contains(textoBuscar) ||
                                             e.Accion.Nombre.Contains(textoBuscar) ||
                                             e.Poblacion.Provincia.Nombre.Contains(textoBuscar) ||
                                             e.Poblacion.Nombre.Contains(textoBuscar));
                                             //Formato.Fecha(e.FechaInicio).Contains(textoBuscar) || Formato.Fecha(e.FechaFin).Contains(textoBuscar));

            return eventos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de eventos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de eventos</returns>
        public IEnumerable<Evento> ObtenerTodos(int pagina, int filas, Func<Evento, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Evento> eventos;
            if (orden == Orden.Ascendente)
                eventos = datos.Eventos.OrderBy(funcOrden);
            else
                eventos = datos.Eventos.OrderByDescending(funcOrden);

            return eventos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de eventos 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de eventos</returns>
        public IEnumerable<Evento> ObtenerTodos()
        {
            return datos.Eventos.ToList();
        }

        /// <summary>
        /// Obtiene una evento a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una evento</returns>
        public Evento ObtenerId(int id)
        {
            return datos.Eventos.FirstOrDefault(m => m.IdEvento == id);
        }

        #endregion

    }
}