﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de foto de evento. Implementa los métodos de la interfaz del servicio de foto
    /// </summary>
    public class FotoService : IFotoService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region Miembros de IBaseService<Foto,int>

        /// <summary>
        /// Guarda los datos de foto
        /// </summary>
        /// <param name="entity">Foto a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Foto entity)
        {
            try
            {
                if (entity.IdFoto == 0)
                {
                    // Si es nuevo
                    datos.AddToFotos(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Fotos.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de Foto
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Foto foto = datos.Fotos.FirstOrDefault(m => m.IdFoto == id);
            if (foto == null)
                return false;

            datos.Fotos.DeleteObject(foto);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Fotos.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de fotos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de las fotos</returns>
        public IEnumerable<Foto> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion, string sentidoOrdenacion,
                                               string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idfoto", "evento.nombre", "nombre", "fecha", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var fotos = (IQueryable<Foto>) datos.Fotos.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                fotos = fotos.Where(f => f.Nombre.Contains(textoBuscar) || f.Descripcion.Contains(textoBuscar) ||
                                    f.Evento.Nombre.Contains(textoBuscar)); // f.Fecha.ToString().Contains(textoBuscar));

            return fotos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de fotos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de las documfotosentos</returns>
        public IEnumerable<Foto> ObtenerTodos(int pagina, int filas, Func<Foto, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Foto> fotos;
            if (orden == Orden.Ascendente)
                fotos = datos.Fotos.OrderBy(funcOrden);
            else
                fotos = datos.Fotos.OrderByDescending(funcOrden);

            return fotos.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de fotos 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de fotos</returns>
        public IEnumerable<Foto> ObtenerTodos()
        {
            return datos.Fotos.ToList();
        }

        /// <summary>
        /// Obtiene un foto a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un foto</returns>
        public Foto ObtenerId(int id)
        {
            return datos.Fotos.FirstOrDefault(m => m.IdFoto == id);
        }

        #endregion

        #region Miembros de IFotoService

        /// <summary>
        /// Obtiene una lista con las fotos por evento
        /// </summary>
        /// <param name="idevento">Identificador del evento</param>
        /// <returns>Devuelve las fotos por evento</returns>
        public IEnumerable<Foto> ObtenerEvento(int idevento)
        {
            return datos.Fotos.Where(f => f.IdEvento.Equals(idevento)).ToList(); 
        }

        #endregion
    }
}