﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de medio de entidad. Implementa los métodos de la interfaz del servicio de medio
    /// </summary>
    public class MedioService : IMedioService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IMedioService Members

        /// <summary>
        /// Guarda los datos de medio
        /// </summary>
        /// <param name="entity">Medio a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Medio entity)
        {
            try
            {
                if (entity.IdMedio == 0)
                {
                    // Si es nuevo
                    datos.AddToMedios(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Medios.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de medio
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Medio medio = datos.Medios.FirstOrDefault(m => m.IdMedio == id);
            if (medio == null)
                return false;

            datos.Medios.DeleteObject(medio);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Medios.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de medios 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de medios</returns>
        public IEnumerable<Medio> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idmedio", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var medios = (IQueryable<Medio>) datos.Medios.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                medios = medios.Where(m => m.Descripcion.Contains(textoBuscar));

            return medios.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de medios 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de medios</returns>
        public IEnumerable<Medio> ObtenerTodos(int pagina, int filas, Func<Medio, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Medio> medios;
            if (orden == Orden.Ascendente)
                medios = datos.Medios.OrderBy(funcOrden);
            else
                medios = datos.Medios.OrderByDescending(funcOrden);

            return medios.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de medios 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de medios</returns>
        public IEnumerable<Medio> ObtenerTodos()
        {
            return datos.Medios.ToList();
        }

        /// <summary>
        /// Obtiene un medio a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un medio</returns>
        public Medio ObtenerId(int id)
        {
            return datos.Medios.FirstOrDefault(m => m.IdMedio == id);
        }

        #endregion
    }
}