﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de organización. Implementa los métodos de la interfaz del servicio de organización
    /// </summary>
    public class OrganizacionService : IOrganizacionService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IOrganizacionService Members

        /// <summary>
        /// Guarda los datos de organización
        /// </summary>
        /// <param name="entity">Organización a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Organizacion entity)
        {
            try
            {
                if (entity.IdOrganizacion == 0)
                {
                    // Si es nuevo
                    datos.AddToOrganizaciones(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Organizaciones.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de organización
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Organizacion org = datos.Organizaciones.FirstOrDefault(m => m.IdOrganizacion == id);
            if (org == null)
                return false;

            datos.Organizaciones.DeleteObject(org);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Organizaciones.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de organizaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de organizaciones</returns>
        public IEnumerable<Organizacion> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                       string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idorganizacion", "tipoorganizacion.descripcion", "nombre", "email", "web"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var orgs = (IQueryable<Organizacion>) datos.Organizaciones.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                orgs = orgs.Where( o => o.Nombre.Contains(textoBuscar) || o.EMail.Contains(textoBuscar) ||
                                   o.Web.Contains(textoBuscar) || o.TipoOrganizacion.Descripcion.Contains(textoBuscar));

            return orgs.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de organizaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de organizaciones</returns>
        public IEnumerable<Organizacion> ObtenerTodos(int pagina, int filas, Func<Organizacion, IComparable> funcOrden,
                                                      Orden orden)
        {
            IEnumerable<Organizacion> orgs;
            if (orden == Orden.Ascendente)
                orgs = datos.Organizaciones.OrderBy(funcOrden);
            else
                orgs = datos.Organizaciones.OrderByDescending(funcOrden);

            return orgs.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de organizaciones 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de organizaciones</returns>
        public IEnumerable<Organizacion> ObtenerTodos()
        {
            return datos.Organizaciones.ToList();
        }

        /// <summary>
        /// Obtiene una organización a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una organización</returns>
        public Organizacion ObtenerId(int id)
        {
            return datos.Organizaciones.FirstOrDefault(m => m.IdOrganizacion == id);
        }

        #endregion
    }
}