﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de participación de eventos. Implementa los métodos de la interfaz del servicio de participación
    /// </summary>
    public class ParticipacionService : IParticipacionService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region Miembros de IBaseService<Participacion,int>

        /// <summary>
        /// Guarda los datos de participación
        /// </summary>
        /// <param name="entity">Participacion a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Participacion entity)
        {
            try
            {
                if (entity.IdParticipacion == 0)
                {
                    // Si es nuevo
                    datos.AddToParticipaciones(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Participaciones.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de participación
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Participacion participación = datos.Participaciones.FirstOrDefault(m => m.IdParticipacion == id);
            if (participación == null)
                return false;

            datos.Participaciones.DeleteObject(participación);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Participaciones.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de participaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de participaciones</returns>
        public IEnumerable<Participacion> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                        string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase) ? sentidoOrdenacion : "asc";
            var columnasValidas = new[] { "idparticipacion", "identidad", "idevento", "idparticipacion", 
                                          "fechainicio", "fechafin", "material" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var participaciones = (IQueryable<Participacion>)datos.Participaciones.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                participaciones = participaciones.Where(p => p.FechaInicio.ToString().Contains(textoBuscar) ||
                                                             p.FechaFin.ToString().Contains(textoBuscar) ||
                                                             p.Material.Contains(textoBuscar));

            return participaciones.Skip((pagina - 1) * filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de participaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de participaciones</returns>
        public IEnumerable<Participacion> ObtenerTodos(int pagina, int filas, Func<Participacion, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Participacion> participaciones;
            if (orden == Orden.Ascendente)
                participaciones = datos.Participaciones.OrderBy(funcOrden);
            else
                participaciones = datos.Participaciones.OrderByDescending(funcOrden);

            return participaciones.Skip((pagina - 1) * filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de participaciones 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de participaciones</returns>
        public IEnumerable<Participacion> ObtenerTodos()
        {
            return datos.Participaciones.ToList();
        }

        /// <summary>
        /// Obtiene una participación a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una participación</returns>
        public Participacion ObtenerId(int id)
        {
            return datos.Participaciones.FirstOrDefault(m => m.IdParticipacion == id);
        }

        #endregion

        #region IParticipacionService Members

        /// <summary>
        /// Obtiene una lista con las participaciones por evento
        /// </summary>
        /// <param name="idevento">Identificador del evento</param>
        /// <returns>Devuelve las participaciones por evento</returns>
        public IEnumerable<Participacion> ObtenerEvento(int idevento)
        {
            return datos.Participaciones.Where(p => p.IdEvento.Equals(idevento)).ToList();
        }

        /// <summary>
        /// Obtiene una lista con las participaciones por entidad
        /// </summary>
        /// <param name="identidad">Identificador de la entidad</param>
        /// <returns>Devuelve las participaciones por entidad</returns>
        public IEnumerable<Participacion> ObtenerEntidad(int identidad)
        {
            return datos.Participaciones.Where(p => p.IdEntidad.Equals(identidad)).ToList();
        }

        #endregion
    }
}