﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de población. Implementa los métodos de la interfaz del servicio de población
    /// </summary>
    public class PoblacionService : IPoblacionService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IPoblacionService Members

        /// <summary>
        /// Guarda los datos de población
        /// </summary>
        /// <param name="entity">Población a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Poblacion entity)
        {
            try
            {
                if (entity.IdPoblacion == 0)
                {
                    // Si es nuevo
                    datos.AddToPoblaciones(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Poblaciones.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de población
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Poblacion poblacion = datos.Poblaciones.FirstOrDefault(m => m.IdPoblacion == id);
            if (poblacion == null)
                return false;

            datos.Poblaciones.DeleteObject(poblacion);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Poblaciones.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de poblaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de poblaciones</returns>
        public IEnumerable<Poblacion> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                    string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idpoblacion", "idprovincia", "nombre", "poblacionseo", "codigopostal"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var poblaciones =
                (IQueryable<Poblacion>) datos.Poblaciones.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                poblaciones =
                    poblaciones.Where(
                        p =>
                        p.Nombre.Contains(textoBuscar) || p.PoblacionSeo.Contains(textoBuscar) ||
                        p.CodigoPostal.Contains(textoBuscar));

            return poblaciones.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de poblaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de poblaciones</returns>
        public IEnumerable<Poblacion> ObtenerTodos(int pagina, int filas, Func<Poblacion, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Poblacion> poblaciones;
            if (orden == Orden.Ascendente)
                poblaciones = datos.Poblaciones.OrderBy(funcOrden);
            else
                poblaciones = datos.Poblaciones.OrderByDescending(funcOrden);

            return poblaciones.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de poblaciones 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de poblaciones</returns>
        public IEnumerable<Poblacion> ObtenerTodos()
        {
            return datos.Poblaciones.ToList();
        }

        /// <summary>
        /// Obtiene una población a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una población</returns>
        public Poblacion ObtenerId(int id)
        {
            return datos.Poblaciones.FirstOrDefault(m => m.IdPoblacion == id);
        }

        /// <summary>
        /// Obtiene una población a partir del nombre
        /// </summary>
        /// <param name="nombre">Nombre a obtener</param>
        /// <returns>Devuelve los datos de una población</returns>
        public IEnumerable<Poblacion> ObtenerNombre(string nombre)
        {
            return datos.Poblaciones.Where(p => p.Nombre.ToLower().Contains(nombre.ToLower())).ToList();
        }

        #endregion

    }
}