﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de provincia. Implementa los métodos de la interfaz del servicio de provincia
    /// </summary>
    public class ProvinciaService : IProvinciaService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IProvinciaService Members

        /// <summary>
        /// Guarda los datos de provincia
        /// </summary>
        /// <param name="entity">Provincia a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Provincia entity)
        {
            try
            {
                if (entity.IdProvincia == 0)
                {
                    // Si es nuevo
                    datos.AddToProvincias(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Provincias.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de provincia
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Provincia provincia = datos.Provincias.FirstOrDefault(m => m.IdProvincia == id);
            if (provincia == null)
                return false;

            datos.Provincias.DeleteObject(provincia);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Provincias.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de provincias 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de provincias</returns>
        public IEnumerable<Provincia> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                    string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idprovincia", "nombre", "provinciaseo", "siglas"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var provincias =
                (IQueryable<Provincia>) datos.Provincias.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                provincias =
                    provincias.Where(
                        p =>
                        p.Nombre.Contains(textoBuscar) || p.Provinciaseo.Contains(textoBuscar) ||
                        p.Siglas.Contains(textoBuscar));

            return provincias.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de provincias 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de provincias</returns>
        public IEnumerable<Provincia> ObtenerTodos(int pagina, int filas, Func<Provincia, IComparable> funcOrden,
                                                   Orden orden)
        {
            IEnumerable<Provincia> provincias;
            if (orden == Orden.Ascendente)
                provincias = datos.Provincias.OrderBy(funcOrden);
            else
                provincias = datos.Provincias.OrderByDescending(funcOrden);

            return provincias.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de provincias 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de provincias</returns>
        public IEnumerable<Provincia> ObtenerTodos()
        {
            return datos.Provincias.ToList();
        }

        /// <summary>
        /// Obtiene una provincia a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una provincia</returns>
        public Provincia ObtenerId(int id)
        {
            return datos.Provincias.FirstOrDefault(m => m.IdProvincia == id);
        }

        #endregion
    }
}