﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de rol de usuario. Implementa los métodos de la interfaz del servicio de rol
    /// </summary>
    public class RolService : IRolService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IRolService Members

        /// <summary>
        /// Guarda los datos de rol
        /// </summary>
        /// <param name="entity">Rol a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Rol entity)
        {
            try
            {
                if (entity.IdRol == 0)
                {
                    // Si es nuevo
                    datos.AddToRoles(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Roles.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de rol
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Rol rol = datos.Roles.FirstOrDefault(m => m.IdRol == id);
            if (rol == null)
                return false;

            datos.Roles.DeleteObject(rol);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Roles.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de roles 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de roles</returns>
        public IEnumerable<Rol> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion, string sentidoOrdenacion,
                                              string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idrol", "codigo", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var roles = (IQueryable<Rol>) datos.Roles.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                roles = roles.Where(r => r.Codigo.Contains(textoBuscar) || r.Descripcion.Contains(textoBuscar));

            return roles.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de roles 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de roles</returns>
        public IEnumerable<Rol> ObtenerTodos(int pagina, int filas, Func<Rol, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Rol> roles;
            if (orden == Orden.Ascendente)
                roles = datos.Roles.OrderBy(funcOrden);
            else
                roles = datos.Roles.OrderByDescending(funcOrden);

            return roles.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de roles 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de roles</returns>
        public IEnumerable<Rol> ObtenerTodos()
        {
            return datos.Roles.ToList();
        }

        /// <summary>
        /// Obtiene un rol a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un rol</returns>
        public Rol ObtenerId(int id)
        {
            return datos.Roles.FirstOrDefault(m => m.IdRol == id);
        }


        /// <summary>
        /// Obtiene los diferentes roles existentes
        /// </summary>
        /// <returns>Devuelve un array con los diferentes roles existentes</returns>
        public string[] GetAllRoles()
        {
            return (from r in datos.Roles select r.Codigo).Distinct().ToArray();
        }

        /// <summary>
        /// Obtiene los roles mayores al rol actual 
        /// </summary>
        /// <param name="roleName">Nombre del rol</param>
        /// <returns>Devuelve un array con los roles</returns>
        public string[] GetRoles(string roleName)
        {
            string[] roles = null;

            // Obtiene el el rol
            Rol rol = datos.Roles.FirstOrDefault(r => r.Codigo.Equals(roleName));

            if (rol != null)
            {
                // Devuelve la lista de roles mayores al rol actual (el menor es el de mayor prioridad)
                roles = (from r in datos.Roles where r.IdRol >= rol.IdRol select r.Codigo).Distinct().ToArray();
            }

            return roles;
        }

        /// <summary>
        /// Obtiene los roles de un usuario
        /// </summary>
        /// <param name="username">Nombre del usuario</param>
        /// <returns>Devuelve un array con los roles de un usuario</returns>
        public string[] GetByUserName(string username)
        {
            string[] roles = null;

            // Obtiene el usuario a partir del nombre de usuario (único)
            Usuario usuario = datos.Usuarios.FirstOrDefault(u => u.NombreUsuario.Equals(username));

            // Si lo encuentra
            if (usuario != null)
            {
                // Devuelve los roles del usuario 
                roles = GetRoles(usuario.Rol.Codigo);
            }

            return roles;
        }

        /// <summary>
        /// Comprueba la existencia de un rol
        /// </summary>
        /// <param name="roleName">Nombre del rol</param>
        /// <returns>Devuelve 'true' si existe el rol</returns>
        public bool RoleExists(string roleName)
        {
            return datos.Roles.Any(r => r.Codigo.Equals(roleName));
        }

        /// <summary>
        /// Comprueba si un usuario pertenece a un rol
        /// </summary>
        /// <param name="username">Nombre del usuario</param>
        /// <param name="roleName">Nombre del rol</param>
        /// <returns>Devuelve 'true' si el usuario tienee el rol</returns>
        public bool IsUserInRole(string username, string roleName)
        {
            // Obtener los roles del usuario
            string[] roles = GetByUserName(username);

            return roles.Any(r => r.Equals(roleName));
        }

        #endregion
    }
}