﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de seguimiento de entidad. Implementa los métodos de la interfaz del servicio de seguimiento
    /// </summary>
    public class SeguimientoService : ISeguimientoService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region Miembros de IBaseService<Seguimiento,int>

        /// <summary>
        /// Guarda los datos de seguimiento
        /// </summary>
        /// <param name="entity">Seguimiento a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Seguimiento entity)
        {
            try
            {
                if (entity.IdSeguimiento == 0)
                {
                    // Si es nuevo
                    datos.AddToSeguimientos(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Seguimientos.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de Seguimiento
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Seguimiento seguimiento = datos.Seguimientos.FirstOrDefault(m => m.IdSeguimiento == id);
            if (seguimiento == null)
                return false;

            datos.Seguimientos.DeleteObject(seguimiento);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Seguimientos.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de seguimientos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de las seguimientos</returns>
        public IEnumerable<Seguimiento> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                    string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase) ? sentidoOrdenacion : "asc";
            var columnasValidas = new[] {"idseguimiento", "identidad", "idusuario", "fecha", "idmedio", 
                                         "idestado", "observaciones"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "fecha";

            // Generamos la consulta
            var seguimientos = (IQueryable<Seguimiento>) datos.Seguimientos.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                seguimientos = seguimientos.Where(s => s.Fecha.ToString().Contains(textoBuscar) ||
                                                       s.Observaciones.Contains(textoBuscar));

            return seguimientos.Skip((pagina - 1) * filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de seguimientos 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de las seguimientos</returns>
        public IEnumerable<Seguimiento> ObtenerTodos(int pagina, int filas, Func<Seguimiento, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Seguimiento> seguimientos;
            if (orden == Orden.Ascendente)
                seguimientos = datos.Seguimientos.OrderBy(funcOrden);
            else
                seguimientos = datos.Seguimientos.OrderByDescending(funcOrden);

            return seguimientos.Skip((pagina - 1) * filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de seguimientos 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de seguimientos</returns>
        public IEnumerable<Seguimiento> ObtenerTodos()
        {
            return datos.Seguimientos.ToList();
        }

        /// <summary>
        /// Obtiene un seguimiento a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un seguimiento</returns>
        public Seguimiento ObtenerId(int id)
        {
            return datos.Seguimientos.FirstOrDefault(s => s.IdSeguimiento == id);
        }

        #endregion

        #region Miembros de ISeguimientoService

        /// <summary>
        /// Obtiene una lista con los seguimientos por entidad
        /// </summary>
        /// <param name="identidad">Identificador de la entidad</param>
        /// <returns>Devuelve los seguimientos por entidad</returns>
        public IEnumerable<Seguimiento> ObtenerEntidad(int identidad)
        {
            return datos.Seguimientos.Where(s => s.IdEntidad.Equals(identidad)).ToList();
        }

        #endregion
    }
}