﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de temática de acción. Implementa los métodos de la interfaz del servicio de temática
    /// </summary>
    public class TematicaService : ITematicaService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region ITematicaService Members

        /// <summary>
        /// Guarda los datos de temática
        /// </summary>
        /// <param name="entity">Temática a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Tematica entity)
        {
            try
            {
                if (entity.IdTematica == 0)
                {
                    // Si es nuevo
                    datos.AddToTematicas(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Tematicas.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de temática
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Tematica tematica = datos.Tematicas.FirstOrDefault(m => m.IdTematica == id);
            if (tematica == null)
                return false;

            datos.Tematicas.DeleteObject(tematica);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Tematicas.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de temáticas 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de temáticas</returns>
        public IEnumerable<Tematica> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                   string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idtematica", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var tematicas =
                (IQueryable<Tematica>) datos.Tematicas.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                tematicas = tematicas.Where(e => e.Descripcion.Contains(textoBuscar));

            return tematicas.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de temáticas 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de temáticas</returns>
        public IEnumerable<Tematica> ObtenerTodos(int pagina, int filas, Func<Tematica, IComparable> funcOrden,
                                                  Orden orden)
        {
            IEnumerable<Tematica> tematicas;
            if (orden == Orden.Ascendente)
                tematicas = datos.Tematicas.OrderBy(funcOrden);
            else
                tematicas = datos.Tematicas.OrderByDescending(funcOrden);

            return tematicas.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de temáticas 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de temáticas</returns>
        public IEnumerable<Tematica> ObtenerTodos()
        {
            return datos.Tematicas.ToList();
        }

        /// <summary>
        /// Obtiene una temática a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de una temática</returns>
        public Tematica ObtenerId(int id)
        {
            return datos.Tematicas.FirstOrDefault(m => m.IdTematica == id);
        }

        #endregion
    }
}