﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de tipo de entidad. Implementa los métodos de la interfaz del servicio de tipo de entidad
    /// </summary>
    public class TipoEntidadService : ITipoEntidadService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region ITipoEntidadService Members

        /// <summary>
        /// Guarda los datos de tipo de entidad
        /// </summary>
        /// <param name="entity">Tipo de entidad a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(TipoEntidad entity)
        {
            try
            {
                if (entity.IdTipoEntidad == 0)
                {
                    // Si es nuevo
                    datos.AddToTiposEntidades(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.TiposEntidades.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de tipo de entidad
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            TipoEntidad tipoentidad = datos.TiposEntidades.FirstOrDefault(m => m.IdTipoEntidad == id);
            if (tipoentidad == null)
                return false;

            datos.TiposEntidades.DeleteObject(tipoentidad);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.TiposEntidades.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de tipos de entidades 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de tipos de entidades</returns>
        public IEnumerable<TipoEntidad> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                      string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idtipoentidad", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var tiposentidades =
                (IQueryable<TipoEntidad>)
                datos.TiposEntidades.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                tiposentidades = tiposentidades.Where(t => t.Descripcion.Contains(textoBuscar));

            return tiposentidades.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de tipos de entidades 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de tipos de entidades</returns>
        public IEnumerable<TipoEntidad> ObtenerTodos(int pagina, int filas, Func<TipoEntidad, IComparable> funcOrden,
                                                     Orden orden)
        {
            IEnumerable<TipoEntidad> tiposentidades;
            if (orden == Orden.Ascendente)
                tiposentidades = datos.TiposEntidades.OrderBy(funcOrden);
            else
                tiposentidades = datos.TiposEntidades.OrderByDescending(funcOrden);

            return tiposentidades.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de tipos de entidades 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de tipos de entidades</returns>
        public IEnumerable<TipoEntidad> ObtenerTodos()
        {
            return datos.TiposEntidades.ToList();
        }

        /// <summary>
        /// Obtiene un tipo de entidad a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un tipo de entidad</returns>
        public TipoEntidad ObtenerId(int id)
        {
            return datos.TiposEntidades.FirstOrDefault(m => m.IdTipoEntidad == id);
        }

        #endregion
    }
}