﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de tipo de organización. Implementa los métodos de la interfaz del servicio de tipo de organización
    /// </summary>
    public class TipoOrganizacionService : ITipoOrganizacionService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region ITipoOrganizacionService Members

        /// <summary>
        /// Guarda los datos de tipo de organización
        /// </summary>
        /// <param name="entity">Tipo de organización a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(TipoOrganizacion entity)
        {
            try
            {
                if (entity.IdTipoOrganizacion == 0)
                {
                    // Si es nuevo
                    datos.AddToTiposOrganizaciones(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.TiposOrganizaciones.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de tipo de organización
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            TipoOrganizacion tipoorg = datos.TiposOrganizaciones.FirstOrDefault(m => m.IdTipoOrganizacion == id);
            if (tipoorg == null)
                return false;

            datos.TiposOrganizaciones.DeleteObject(tipoorg);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.TiposOrganizaciones.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de tipos de organizaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de tipos de organizaciones</returns>
        public IEnumerable<TipoOrganizacion> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                           string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idtipoorganizacion", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var tiposorgs =
                (IQueryable<TipoOrganizacion>)
                datos.TiposOrganizaciones.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                tiposorgs = tiposorgs.Where(t => t.Descripcion.Contains(textoBuscar));

            return tiposorgs.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de tipos de organizaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de tipos de organizaciones</returns>
        public IEnumerable<TipoOrganizacion> ObtenerTodos(int pagina, int filas,
                                                          Func<TipoOrganizacion, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<TipoOrganizacion> tiposorgs;
            if (orden == Orden.Ascendente)
                tiposorgs = datos.TiposOrganizaciones.OrderBy(funcOrden);
            else
                tiposorgs = datos.TiposOrganizaciones.OrderByDescending(funcOrden);

            return tiposorgs.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de tipos de organizaciones 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de tipos de organizaciones</returns>
        public IEnumerable<TipoOrganizacion> ObtenerTodos()
        {
            return datos.TiposOrganizaciones.ToList();
        }

        /// <summary>
        /// Obtiene un tipo de organización a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un tipo de organización</returns>
        public TipoOrganizacion ObtenerId(int id)
        {
            return datos.TiposOrganizaciones.FirstOrDefault(m => m.IdTipoOrganizacion == id);
        }

        #endregion
    }
}