﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de tipo de participación. Implementa los métodos de la interfaz del servicio de tipo de participación
    /// </summary>
    public class TipoParticipacionService : ITipoParticipacionService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region ITipoParticipacionService Members

        /// <summary>
        /// Guarda los datos de tipo de participación
        /// </summary>
        /// <param name="entity">Tipo de participación a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(TipoParticipacion entity)
        {
            try
            {
                if (entity.IdTipoParticipacion == 0)
                {
                    // Si es nuevo
                    datos.AddToTiposParticipaciones(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.TiposParticipaciones.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de tipo de participación
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            TipoParticipacion tipoparticipacion =
                datos.TiposParticipaciones.FirstOrDefault(m => m.IdTipoParticipacion == id);
            if (tipoparticipacion == null)
                return false;

            datos.TiposParticipaciones.DeleteObject(tipoparticipacion);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.TiposEntidades.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de tipos de participaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de tipos de participaciones</returns>
        public IEnumerable<TipoParticipacion> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                            string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] {"idtipoparticipacion", "descripcion"};
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "descripcion";

            // Generamos la consulta
            var tiposparticipaciones =
                (IQueryable<TipoParticipacion>)
                datos.TiposParticipaciones.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                tiposparticipaciones = tiposparticipaciones.Where(t => t.Descripcion.Contains(textoBuscar));

            return tiposparticipaciones.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de tipos de participaciones 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de tipos de participaciones</returns>
        public IEnumerable<TipoParticipacion> ObtenerTodos(int pagina, int filas,
                                                           Func<TipoParticipacion, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<TipoParticipacion> tiposparticipaciones;
            if (orden == Orden.Ascendente)
                tiposparticipaciones = datos.TiposParticipaciones.OrderBy(funcOrden);
            else
                tiposparticipaciones = datos.TiposParticipaciones.OrderByDescending(funcOrden);

            return tiposparticipaciones.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de tipos de participaciones 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de tipos de participaciones</returns>
        public IEnumerable<TipoParticipacion> ObtenerTodos()
        {
            return datos.TiposParticipaciones.ToList();
        }

        /// <summary>
        /// Obtiene un tipo de participación a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un tipo de participación</returns>
        public TipoParticipacion ObtenerId(int id)
        {
            return datos.TiposParticipaciones.FirstOrDefault(m => m.IdTipoParticipacion == id);
        }

        #endregion
    }
}