﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using GPAS.Models.Interfaces;

namespace GPAS.Models.Services
{
    /// <summary>
    /// Clase de servicio de datos de usuario. Implementa los métodos de la interfaz del servicio de usuario
    /// </summary>
    public class UsuarioService : IUsuarioService
    {
        private readonly GPASEntities datos = new GPASEntities();

        #region IUsuarioService Members

        /// <summary>
        /// Guarda los datos de usuario
        /// </summary>
        /// <param name="entity">Usuario a guardar</param>
        /// <returns>Devuelve 'true' si se guarda correctamente</returns>
        public bool Guardar(Usuario entity)
        {
            try
            {
                if (entity.IdUsuario == 0)
                {
                    // Si es nuevo
                    datos.AddToUsuarios(entity);
                }
                else
                {
                    // Si es una modificación
                    datos.Usuarios.Attach(entity);
                    datos.ObjectStateManager.ChangeObjectState(entity, EntityState.Modified);
                }
                datos.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                return false;
            }
        }

        /// <summary>
        /// Borra los datos de usuario
        /// </summary>
        /// <param name="id">Identificador a borrar</param>
        /// <returns>Devuelve 'true' si se borra correctamente</returns>
        public bool Borrar(int id)
        {
            Usuario usuario = datos.Usuarios.FirstOrDefault(m => m.IdUsuario == id);
            if (usuario == null)
                return false;

            datos.Usuarios.DeleteObject(usuario);
            datos.SaveChanges();
            return true;
        }

        /// <summary>
        /// Cuenta el número de elementos existente
        /// </summary>
        /// <returns>Devuelve el número de elementos</returns>
        public int Contar()
        {
            return datos.Usuarios.Count();
        }

        /// <summary>
        /// Obtiene una lista filtrada, paginada y ordenada con los datos de usuarios 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="columnaOrdenacion">Nombre del campo de ordenación</param>
        /// <param name="sentidoOrdenacion">Sentido de la ordenación (asc=Ascendente, des=Descendente)</param>
        /// <param name="textoBuscar">Texto para filtrar los datos</param>
        /// <returns>Devuelve una lista con los datos de usuarios</returns>
        public IEnumerable<Usuario> ObtenerFiltro(int pagina, int filas, string columnaOrdenacion,
                                                  string sentidoOrdenacion, string textoBuscar)
        {
            // Comprobamos los datos de entrada
            if (pagina < 1) pagina = 1;
            if (filas < 1) filas = 10;
            sentidoOrdenacion = sentidoOrdenacion.Equals("desc", StringComparison.CurrentCultureIgnoreCase)
                                    ? sentidoOrdenacion
                                    : "asc";
            var columnasValidas = new[] { "idusuario", "delegacion.nombre", "nombreusuario", "nombre", "apellidos", "email", "activo" };
            if (!columnasValidas.Contains(columnaOrdenacion.ToLower())) columnaOrdenacion = "nombre";

            // Generamos la consulta
            var usuarios =
                (IQueryable<Usuario>) datos.Usuarios.OrderBy("it." + columnaOrdenacion + " " + sentidoOrdenacion);

            // Aplicamos el filtro
            if (!string.IsNullOrWhiteSpace(textoBuscar))
                usuarios = usuarios.Where( u => u.Delegacion.Nombre.Contains(textoBuscar) || u.NombreUsuario.Contains(textoBuscar) ||
                                                u.Nombre.Contains(textoBuscar) || u.Apellidos.Contains(textoBuscar) ||
                                                u.EMail.Contains(textoBuscar));

            return usuarios.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una lista paginada y ordenada con los datos de usuarios 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <param name="funcOrden">Funcion para la ordenación</param>
        /// <param name="orden">Sentido de la ordenación (Ascendente, Descendente)</param>
        /// <returns>Devuelve una lista con todos los datos de usuarios</returns>
        public IEnumerable<Usuario> ObtenerTodos(int pagina, int filas, Func<Usuario, IComparable> funcOrden, Orden orden)
        {
            IEnumerable<Usuario> usuarios;

            // Generamos la consulta
            if (orden == Orden.Ascendente)
                usuarios = datos.Usuarios.OrderBy(funcOrden);
            else
                usuarios = datos.Usuarios.OrderByDescending(funcOrden);

            return usuarios.Skip((pagina - 1)*filas).Take(filas);
        }

        /// <summary>
        /// Obtiene una una lista con los datos de usuarios 
        /// </summary>
        /// <returns>Devuelve una lista con todos los datos de usuarios</returns>
        public IEnumerable<Usuario> ObtenerTodos()
        {
            return datos.Usuarios.ToList();
        }

        /// <summary>
        /// Obtiene un usuario a partir del identificador
        /// </summary>
        /// <param name="id">Identificador a obtener</param>
        /// <returns>Devuelve los datos de un usuario</returns>
        public Usuario ObtenerId(int id)
        {
            return datos.Usuarios.FirstOrDefault(u => u.IdUsuario == id);
        }

        /// <summary>
        /// Obtiene una lista con los datos de usuarios por nombre
        /// </summary>
        /// <param name="username">Nombre del usuario</param>
        /// <returns>Devuelve los datos de usuarios por nombre</returns>
        public IEnumerable<Usuario> ObtenerNombre(string username)
        {
            return datos.Usuarios.Where(u => u.NombreUsuario.Equals(username)).ToList();
        }

        /// <summary>
        /// Obtiene una lista con los datos de usuarios por correo electrónico
        /// </summary>
        /// <param name="email">Correo electrónico de usuario</param>
        /// <returns>Devuelve los datos de usuarios por correo electrónico</returns>
        public IEnumerable<Usuario> ObtenerEmail(string email)
        {
            return datos.Usuarios.Where(u => u.EMail.Equals(email)).ToList();
        }

        /// <summary>
        /// Obtiene el primer usuario por nombre
        /// </summary>
        /// <param name="username">Nombre del usuario</param>
        /// <returns>Devuelve los datos del primer usuario por nombre</returns>
        public Usuario ObtenerPrimerNombreName(string username)
        {
            return datos.Usuarios.FirstOrDefault(u => u.NombreUsuario.Equals(username));
        }

        /// <summary>
        /// Obtiene el primer usuario por correo electrónico
        /// </summary>
        /// <param name="email">Correo electrónico de usuario</param>
        /// <returns>Devuelve los datos del primer usuario por correo electrónico</returns>
        public Usuario ObtenerPrimerEmail(string email)
        {
            return datos.Usuarios.FirstOrDefault(u => u.EMail.Equals(email));
        }

        /// <summary>
        /// Valida que el nombre y la contraseña del usuarios sean válidos
        /// </summary>
        /// <param name="username">Nombre del usuario</param>
        /// <param name="password">Contraseña del usuario</param>
        /// <returns>Devuelve los datos del primer usuario por correo electrónico</returns>
        public bool ValidaUsusario(string username, string password)
        {
            return datos.Usuarios.Any(u => u.NombreUsuario.Equals(username) && u.Contrasena.Equals(password));
        }

        /// <summary>
        /// Comprueba la existencia de un usuario por el nombre
        /// </summary>
        /// <param name="username">Nombre del usuario</param>
        /// <returns>Devuelve 'true' si existe el usuario</returns>
        public bool ExisteNombre(string username)
        {
            return datos.Usuarios.FirstOrDefault(u => u.NombreUsuario.Equals(username)) == null;
        }

        #endregion

        /// <summary>
        /// Obtiene una una lista paginada con los datos de usuarios 
        /// </summary>
        /// <param name="pagina">Número de pagina</param>
        /// <param name="filas">Número de filas</param>
        /// <returns>Devuelve una lista con los datos de usuarios de la página</returns>
        public IEnumerable<Usuario> ObtenerTodos(int pagina, int filas)
        {
            return datos.Usuarios.Skip((pagina - 1)*filas).Take(filas);
        }
    }
}