﻿using System;
using System.Collections.Generic;
using System.Web.Security;
using GPAS.Models;
using GPAS.Models.Interfaces;
using GPAS.Models.Services;

namespace GPAS.Security
{
    /// <summary>   
    /// Proveedor de usuarios. Se crea un MembershipProvider personalizado.
    /// Se usan repositorios, basta con inyectarlos vía propiedades Para Unity: 
    /// Se declaran las propiedades del tipo repositorio públicas con getter y setter y decoradas con [Dependency()]
    /// Desde global.asax se obtiene el Membership Provider y se llama a .BuildUp() con el memberhsìp provider
    /// </summary>
    public class GPASMembershipProvider : MembershipProvider
    {
        private const int MinReqPwdLen = 5;             // Tamaño minimo de la longitud de la contraseña
        private const int MaxInvPwdAttempts = 3;        // Numero máximo de intentos con contraseña inválida

        private readonly IUsuarioService srvUsuario;    // Instancia de servicio del usuario 

        public GPASMembershipProvider()
        {
            srvUsuario = new UsuarioService();
            ApplicationName = "GPAS";
        }

        public GPASMembershipProvider(IUsuarioService srvUsuario)
        {
            this.srvUsuario = srvUsuario; //  as UsuarioService;
            ApplicationName = "GPAS";
        }

        public override string ApplicationName { get; set; }

        public override bool EnablePasswordReset
        {
            get { return false; }
        }

        public override bool EnablePasswordRetrieval
        {
            get { return false; }
        }

        public override int MaxInvalidPasswordAttempts
        {
            get { return MaxInvPwdAttempts; }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get { throw new NotImplementedException(); }
        }

        public override int MinRequiredPasswordLength
        {
            get { return MinReqPwdLen; }
        }

        public override int PasswordAttemptWindow
        {
            get { throw new NotImplementedException(); }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get { throw new NotImplementedException(); }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { throw new NotImplementedException(); }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get { throw new NotImplementedException(); }
        }

        public override bool RequiresUniqueEmail
        {
            get { throw new NotImplementedException(); }
        }

        public override MembershipUser CreateUser(string username, string password, string email,
                                                  string passwordQuestion,
                                                  string passwordAnswer, bool isApproved, object providerUserKey,
                                                  out MembershipCreateStatus status)
        {
            bool cancel = false;

            var e = new ValidatePasswordEventArgs(username, password, true);
            OnValidatingPassword(e);
            cancel = e.Cancel;
            if (cancel)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            // Crea un nuevo usuario
            var usuario = new Usuario();
            usuario.NombreUsuario = username;
            usuario.Contrasena = password;

            // Guarda el usuario en la base de datos
            if (srvUsuario.Guardar(usuario))
                status = MembershipCreateStatus.Success;
            else
                status = MembershipCreateStatus.InvalidUserName;

            return new MembershipUser("GPASMembershipProvider", username, providerUserKey, email, passwordQuestion,
                                      string.Empty, isApproved, false, DateTime.Now, DateTime.Now, DateTime.Now,
                                      DateTime.Now, DateTime.Now);
        }

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            Usuario usuario = srvUsuario.ObtenerPrimerNombreName(username);
            return usuario != null
                       ? new MembershipUser("GPASMembershipProvider", usuario.NombreUsuario, usuario.IdUsuario,
                                            usuario.EMail, string.Empty, string.Empty, true, !usuario.Activo,
                                            DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now)
                       : null;
        }

        public override bool ValidateUser(string username, string password)
        {
            return srvUsuario.ValidaUsusario(username, password);
        }

        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            throw new NotImplementedException();
        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password,
                                                             string newPasswordQuestion, string newPasswordAnswer)
        {
            throw new NotImplementedException();
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize,
                                                                  out int totalRecords)
        {
            var muc = new MembershipUserCollection();

            IEnumerable<Usuario> usuarios = srvUsuario.ObtenerEmail(emailToMatch);
            foreach (Usuario usuario in usuarios)
            {
                muc.Add(new MembershipUser("GPASMembershipProvider", usuario.NombreUsuario, usuario.IdUsuario,
                                           usuario.EMail, string.Empty, string.Empty, true, !usuario.Activo,
                                           DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now));
            }
            totalRecords = muc.Count;

            return muc;
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize,
                                                                 out int totalRecords)
        {
            var muc = new MembershipUserCollection();

            IEnumerable<Usuario> usuarios = srvUsuario.ObtenerNombre(usernameToMatch);
            foreach (Usuario usuario in usuarios)
            {
                muc.Add(new MembershipUser("GPASMembershipProvider", usuario.NombreUsuario, usuario.IdUsuario,
                                           usuario.EMail, string.Empty, string.Empty, true, !usuario.Activo,
                                           DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now));
            }
            totalRecords = muc.Count;

            return muc;
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            var muc = new MembershipUserCollection();

            // Establecemos la función de ordenación y el orden actual    
            string sort = "IdUsuario";
            Func<Usuario, IComparable> funcOrden =
                sort == "Apellidos"
                    ? a => a.Apellidos
                    : (Func<Usuario, IComparable>) (a => a.IdUsuario);
            Orden ordenacion = Orden.Ascendente;

            IEnumerable<Usuario> usuarios = srvUsuario.ObtenerTodos(pageIndex, pageSize, funcOrden, ordenacion);
            foreach (Usuario usuario in usuarios)
            {
                muc.Add(new MembershipUser("GPASMembershipProvider", usuario.NombreUsuario, usuario.IdUsuario,
                                           usuario.EMail, string.Empty, string.Empty, true, !usuario.Activo,
                                           DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now));
            }
            totalRecords = muc.Count;

            return muc;
        }

        public override int GetNumberOfUsersOnline()
        {
            throw new NotImplementedException();
        }

        public override string GetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            Usuario usuario = srvUsuario.ObtenerId((int) providerUserKey);
            return usuario != null
                       ? new MembershipUser("GPASMembershipProvider", usuario.NombreUsuario, usuario.IdUsuario,
                                            usuario.EMail, string.Empty, string.Empty, true, !usuario.Activo,
                                            DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now, DateTime.Now)
                       : null;

            throw new NotImplementedException();
        }

        public override string GetUserNameByEmail(string email)
        {
            Usuario usuario = srvUsuario.ObtenerPrimerEmail(email);
            return usuario != null ? usuario.NombreUsuario : null;
        }

        public override string ResetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override bool UnlockUser(string userName)
        {
            throw new NotImplementedException();
        }

        public override void UpdateUser(MembershipUser user)
        {
            throw new NotImplementedException();
        }
    }
}