﻿using System;
using System.ComponentModel.DataAnnotations;
using System.Globalization;
using System.Web;

namespace GPAS.Validators
{
    public class CustomValidators
    {
        #region ExactNumberOfDigitsAttribute

        public class ExactNumberOfDigitsAttribute : ValidationAttribute
        {
            public ExactNumberOfDigitsAttribute(int number)
            {
                Format = String.Empty;
                NumberOfDigits = number;
            }

            public string Format { get; set; }
            public int NumberOfDigits { get; set; }

            /// <summary>
            /// Reemplaza el método IsValid de la clase ValidationAttribute
            /// <see cref="http://msdn.microsoft.com/en-us/library/cc679289.aspx"/>
            /// </summary>
            /// <param name="value">el valor que estamos validando</param>
            /// <returns></returns>
            public override bool IsValid(object value)
            {
                Format = (String) value;
                bool validationResult = true;
                if (Format.Length != NumberOfDigits) validationResult = false;
                return validationResult;
            }
        }

        #endregion

        #region PhoneNumberFormatAttribute

        /// <summary>
        /// Clase para validar un número de teléfono en nuestro modelo MVC que hereda de ValidationAttribute. Lo marcamos con el
        /// atributo UsageAttribute <see cref="http://msdn.microsoft.com/en-us/library/tw5zxet9(v=VS.100).aspx"/> 
        /// para indicar al compilador que sólo se puede utilizar en propiedades o campos
        /// </summary>
        [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
        public sealed class PhoneNumberFormatAttribute : ValidationAttribute
        {
            public PhoneNumberFormatAttribute(string format)
            {
                Format = format;
            }

            /// <summary>
            /// our format value
            /// </summary>
            public string Format { get; set; }

            /// <summary>
            /// override the IsValid method of the ValidationAttribute class
            /// <see cref="http://msdn.microsoft.com/en-us/library/cc679289.aspx"/>
            /// </summary>
            /// <param name="value">the value we're validating</param>
            /// <returns></returns>
            public override bool IsValid(object value)
            {
                var phone = (String) value;
                bool validationResult = true;

                if (Format != null) validationResult = DoesMatchFormat(Format, phone);

                return validationResult;
            }

            /// <summary>
            /// Método para comprobar si el valor proporcionado coincide con nuestro formato
            /// </summary>
            /// <param name="format">el formato a comprobar (ej.; 999-99999)</param>
            /// <param name="value">el valor a comprobar</param>
            /// <returns>retorna verdadero si esta vacia o concuerda con el formato</returns>
            internal bool DoesMatchFormat(string format, string value)
            {
                // Si el valor esta vacio retrona true
                if (string.IsNullOrEmpty(value)) return true;

                // Si no tiene formato o el valor no tiene la misma longitud retorna false
                if (string.IsNullOrEmpty(format) || format.Length != value.Trim().Length) return false;

                for (int i = 0; i < value.Length; i++)
                {
                    if (format[i] == 'd' && char.IsDigit(value[i]) == false)
                        // Se espera un numero por lo que retorna false
                        return false;

                    if (format[i] == '-' && value[i] != '-')
                        // Se espera un carácter como '-' o ')' por lo que retorna false
                        return false;
                }

                return true;
            }

            /// <summary>
            /// Reemplaza el método FormatErrorMessage de la clase ValidationAttribute
            /// <see cref="http://msdn.microsoft.com/en-us/library/system.componentmodel.dataannotations.validationattribute.formaterrormessage.aspx"/>
            /// </summary>
            /// <param name="name"></param>
            /// <returns></returns>
            public override string FormatErrorMessage(string name)
            {
                return String.Format(CultureInfo.CurrentCulture, ErrorMessageString, name, Format);
            }
        }

        #endregion

        #region FileAttribute

        /*
            public class FileAttribute : ValidationAttribute
            {
                public int MaxContentLength = int.MaxValue;
                public string[] AllowedFileExtensions;
                public string[] AllowedContentTypes;

                public override bool IsValid(object value)
                {
                    var file = value as HttpPostedFileBase;

                    if (file == null)
                        return true;

                    if (file.ContentLength > MaxContentLength)
                    {
                        ErrorMessage = "El fichero es demasiado grande, el máximo permitido es: {0} KB".FormatWith(MaxContentLength / 1024);
                        return false;
                    }

                    if (AllowedFileExtensions != null)
                    {
                        if (!AllowedFileExtensions.Contains(file.FileName.Substring(file.FileName.LastIndexOf('.'))))
                        {
                            ErrorMessage = "Por favor, suba ficheros de tipo: " + string.Join(", ", AllowedFileExtensions);
                            return false;
                        }
                    }

                    if (AllowedContentTypes != null)
                    {
                        if (!AllowedContentTypes.Contains(file.ContentType))
                        {
                            ErrorMessage = "Por favor, suba ficheros de tipo: " + string.Join(", ", AllowedContentTypes);
                            return false;
                        }
                    }

                    return true;
                }
            }
            */

        #endregion
    }
}